<?php
/**
 * YANA Commandline interface
 *
 * This simple program is used to trigger the YANA framework from
 * the command line, for example via a cron-job.
 *
 * To run this script use the following syntax:
 * php cli.php "arg1=value 1" "arg2=value 2" ...
 *
 * Note: errors are printed to a log file (cache/error.log).
 *
 * Note: as of the web-interface the argument "action" is mandatory.
 * The arguments are lower-cased before being processed.
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**#@+
 * @ignore
 */
require_once('library.php');
/**#@-*/

/* prepare input arguments */
$argv = $_SERVER['argv'];
array_shift($argv);

/** BIRTH: $_argv
 * temporary var of type array
 */
$_argv = array();
/** BIRTH: $_arg
 * temporary var of type string
 */
$_arg = "";

while (!empty($argv))
{
    $_arg = array_shift($argv);
    /** BIRTH: $m */
    if (preg_match('/^(\w[\w\/-_]*)=(.*)/', $_arg, $m)) {
        $_argv[strtolower($m[1])] = $m[2];
    } else {
        trigger_error("Malformed argument: \"${arg}\". Expected syntax \"arg1=value 1\".", E_USER_ERROR);
        exit(1);
    }
    /** EOL: $m */
    unset($m);
}
$argv = $_argv;

/** EOL: $_arg */
unset($_arg);
/** EOL: $_argv */
unset($_argv);

if (!isset($argv['action'])) {
    trigger_error("Missing argument: \"action\". This argument is mandatory. Syntax: \"action=foo\"", E_USER_ERROR);
    exit(1);
}

/* Initialize the framework */
ErrorUtility::setErrorReporting(YANA_ERROR_LOG);
$YANA = new Yana("config/system.config", $argv);

/* Output a standard header */
print "
---------------------------------
| Yana - command line interface |
---------------------------------

Running:  ".$_SERVER['argv'][0]."
Time:     ".date('r')."
Arguments:
".print_r($argv, true)."\n";

$YANA->handle($argv['action'], $argv);
foreach ($YANA->log as $log)
{
    switch (strtolower(get_class($log)))
    {
        case 'log':  case 'message':
            trigger_error($log->getMessage(), E_USER_NOTICE);
        break;
        case 'alert':
            trigger_error($log->getMessage(), E_USER_NOTICE);
        break;
        case 'warning':
            trigger_error($log->getMessage(), E_USER_WARNING);
        break;
        case 'error':
            trigger_error($log->getMessage(), E_USER_ERROR);
        break;
        default:
            trigger_error($log->getMessage(), E_USER_WARNING);
        break;
    }
}

if ($YANA->getVar('SETTINGS.LAST_RESULT') === true) {
    print 'Execution finished successfully.';
    exit(0);
} else {
    print 'Execution finished with errors. See the error log for details. (default output path is: cache/error.log)';
    trigger_error('Execution finished with errors.', E_USER_WARNING);
    exit(2);
}
?>