<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * @ignore
 */

/**
 * utility DisplayUtility
 *
 * This is a utility class. It encapsulates extensions to use with
 * the smarty temlate engine.
 *
 * {@internal
 *
 * Additional smarty functions and modifiers are documented elsewhere.
 * This file should be ignored in API-documentation.
 *
 * }}
 *
 * @static
 * @access      public
 * @package     yana
 * @subpackage  core
 *
 * @ignore
 */
class DisplayUtility extends Utility
{

    /**
     * toTimestamp
     *
     * converts input created by usage of the smarty functions
     * "html_select_time" and "html_select_date" to a UNIX-Timestamp
     *
     * @access  public
     * @static
     * @param   string  $prefix
     * @param   array   $ARGS
     * @return  int
     */
    function toTimestamp($prefix, $ARGS)
    {
        $prefix = (string) $prefix;
        return mktime((int) @$ARGS[$prefix.'hour'], (int) @$ARGS[$prefix.'minute'], (int) @$ARGS[$prefix.'second'], (int) @$ARGS[$prefix.'month'], (int) @$ARGS[$prefix.'day'], (int) @$ARGS[$prefix.'year']);
    }

    /**
     * smarty processor htmlPostProcessor
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function htmlPostProcessor($source, &$smarty)
    {
        if (!is_string($source)) {
            return $source;
        }
        global $lcn_note, $instance_id, $YANA;

        $permission = $YANA->getVar('PERMISSION');

        /* Create field for internal form validation */
        if (!is_int($permission) || $permission < 1) {
            $source = str_replace("</form>", "<span class=\"yana_button\"><input type=\"text\" name=\"yana_url\"></span>\n</form>", $source);
        }

        /* Instance Management */
        if (isset($YANA))  {
            $lcn_key = $YANA->getVar("LICENCE.KEY");
            if (!empty($lcn_key)) {
                $a = $YANA->getVar("LICENCE");
                $lcn_note = Layout::fillTemplate($lcn_note, $a);
                $source = preg_replace("/<\/body>/i", $lcn_note."</body>", $source);
            } else {
                $source = preg_replace("/<\/body>/i", $instance_id."</body>", $source);
            }
        } else {
            $source = preg_replace("/<\/body>/i", $instance_id."</body>", $source);
        }

        return $source;
    }

    /**
     * smarty processor processLanguageInstructions
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function processLanguageInstructions($source, &$smarty)
    {
        $source = preg_replace("/".DisplayUtility::_regExpToken("(\\\$LANGUAGE\..*)")."/Us", YANA_LEFT_DELIMITER."\\1|embeddedTags".YANA_RIGHT_DELIMITER, $source);
        return $source;
    }

    /**
     * smarty processor htmlPreProcessor
     *
     * @access  public
     * @static
     * @param   string  $source
     * @param   Smarty  &$templateClass
     * @return  string
     */
    function htmlPreProcessor($source, &$templateClass)
    {
        if (!is_string($source)) {
            return $source;
        }
        global $YANA;

        /* funcionality of 'include' function has been moved to 'import', so ignore the original */
        $source = preg_replace("/".DisplayUtility::_regExpToken("include .*")."/Usi", "", $source);
        /* deactivate 'include_php' for security reasons */
        $source = preg_replace("/".DisplayUtility::_regExpToken("include_php .*")."/Usi", "", $source);
        /* deactivate 'php' for security reasons */
        $source = preg_replace("/".DisplayUtility::_regExpToken("php").".*".DisplayUtility::_regExpToken("\/php")."/Usi", "", $source);
        /* reformat abbreviated %FOO% tags to [%$FOO%] */
        $source = preg_replace("/".toRegExp(YANA_UNI_DELIMITER)."(([A-Z]|\d|_|-|\.)+)".toRegExp(YANA_UNI_DELIMITER)."/", DisplayUtility::_varToken("\\1"), $source);

        $basedir = $templateClass->get_template_vars('BASEDIR');
        /* settype to STRING */
        $basedir = (string) $basedir;
        $is_include = $templateClass->get_template_vars('FILE_IS_INCLUDE');
        if ($is_include) {
            $source = preg_replace("/^.*<body(.*)>(.*)<\/body>.*$/Usi", "<div\\1>\\2</div>", $source);
        }

        if (!empty($basedir)) {
            preg_match_all('/('.DisplayUtility::_regExpToken(')import\s+(?:preparser\s+|)file="(\S*)(".*').')/Ui', $source, $match2);
            for ($i = 0; $i < count($match2[0]); $i++)
            {
                if (preg_match('/\sliteral\s/i' ,$match2[0][$i])) {
                    $source = preg_replace('/'.toRegExp($match2[0][$i]).'/i', preg_replace("/\sliteral\s/i", " ", $match2[0][$i]), $source);
                } else if (preg_match('/\spreparser\s/i' ,$match2[0][$i])) {
                    $source = preg_replace('/'.toRegExp($match2[0][$i]).'/i', preg_replace("/.*<body[^>]*>(.*)<\/body>.*/si", "\\1", implode("", file($basedir.$match2[2][$i]))), $source);
                } else {
                    $source = str_replace($match2[0][$i], $match2[1][$i].'import file="'.$basedir.$match2[2][$i].$match2[3][$i], $source);
                }
            }

            preg_match_all('/('.DisplayUtility::_regExpToken(')insert\s+file="(\S*)(".*').')/Ui', $source, $match2);
            for ($i = 0; $i < count($match2[0]); $i++)
            {
                    $source = preg_replace('/'.toRegExp($match2[0][$i]).'/i', $match2[1][$i].'insert file="'.$basedir.$match2[2][$i].$match2[3][$i], $source);
            }

            preg_match_all('/ background\s*=\s*"(\S*)"/i', $source, $match2);
            for ($i = 0; $i < count($match2[1]); $i++)
            {
                if (!preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace('/ background\s*=\s*"'.toRegExp($match2[1][$i]).'"/i', ' background="'.$basedir.$match2[1][$i].'"', $source);
                }
            }
            preg_match_all('/ src\s*=\s*"(\S*)"/i', $source, $match2);
            for ($i = 0; $i < count($match2[1]); $i++)
            {
                if (!preg_match('/ src\s?=\s?"\S*"/i', $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace('/ src\s*=\s*"'.toRegExp($match2[1][$i]).'"/i', ' src="'.$basedir.$match2[1][$i].'"', $source);
                }
            }
            preg_match_all("/\.src\s*=\s*'(\S*)'/i", $source, $match2);
            for ($i = 0; $i < count($match2[1]); $i++)
            {
                if (!preg_match("/\.src\s*=\s*'\S*'/i", $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace("/\.src\s?=\s?'".toRegExp($match2[1][$i])."'/i", ".src='".$basedir.$match2[1][$i]."'", $source);
                }
            }
            preg_match_all("/ url\(\"([^\"]*)\"\)/i", $source, $match2);
            for ($i = 0; $i < count($match2[1]); $i++)
            {
                if (!preg_match("/ url\(\"[^\"]*\"\)/i", $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace("/ url\(\"".toRegExp($match2[1][$i])."\"\)/i", " url(\"".$basedir.$match2[1][$i]."\")", $source);
                }
            }
            preg_match_all("/ url\('([^']*)'\)/i", $source, $match2);
            for ($i = 0; $i < count($match2[1]); $i++)
            {
                if (!preg_match("/ url\('[^']*'\)/i", $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace("/ url\('".toRegExp($match2[1][$i])."'\)/i", " url('".$basedir.$match2[1][$i]."')", $source);
                }
            }
            preg_match_all('/ href\s*=\s*"([^"]*)"/i', $source, $match2);
            for ($i = 0; $i < count($match2[1]); $i++)
            {
                if (!preg_match('/ href\s*=\s*"[^"]*"/i', $match2[1][$i]) && !preg_match('/^javascript:/i', $match2[1][$i]) && !preg_match('/^\&\#109\;\&\#97\;\&\#105\;\&\#108\;\&\#116\;\&\#111\;\&\#58\;\S*/i', $match2[1][$i]) && !preg_match('/^mailto:\S*/i', $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace('/ href\s?=\s?"'.toRegExp($match2[1][$i]).'"/i', ' href="'.$basedir.$match2[1][$i].'"', $source);
                }
            }

        }

        return DisplayUtility::scanForAt($source);
    }

    /**
     * smarty outputfilter outputfilter
     *
     * @access  public
     * @static
     * @param   string  $source
     * @param   Smarty  &$templateClass
     * @return  string
     */
    function outputFilter($source, &$templateClass)
    {
        $basedir = $templateClass->get_template_vars('BASEDIR');

        /* add meta data to header */
        if (strpos($source, '</head>') > -1) {

            /* import stylesheets */
            $style_list = $templateClass->get_template_vars('SYSTEM_STYLE');
            assert('is_null($style_list) || is_array($style_list);');
            if (is_array($style_list) && count($style_list) > 0) {
                $style_list = array_reverse($style_list, true);
                assert('!isset($stylesheet); /* cannot redeclare variable $stylesheet */');
                foreach ($style_list as $stylesheet)
                {
                    $source = preg_replace('/^(\s*<head[^>]*>)/m', '${1}'."\n".'        '.DisplayUtility::css($stylesheet), $source);
                }
                unset($stylesheet);
            }

            /* import javascripts */
            $script_list = $templateClass->get_template_vars('SYSTEM_SCRIPT');

            assert('is_null($script_list) || is_array($script_list);');
            if (is_array($script_list) && count($script_list) > 0) {
                assert('!isset($script); /* cannot redeclare variable $script */');
                foreach ($script_list as $script)
                {
                    $source = preg_replace('/^(\s*<\/head>)/m', '        '.DisplayUtility::script($script)."\n".'${1}', $source);
                }
                unset($script);
            }

            /* import microsummaries */
            $summary_list = $templateClass->get_template_vars('MICROSUMMARY');

            assert('is_null($summary_list) || is_array($summary_list);');
            if (is_array($summary_list) && count($summary_list) > 0) {
                assert('!isset($summary); /* cannot redeclare variable $summary */');
                foreach ($summary_list as $summary)
                {
                    $source = preg_replace('/^(\s*<\/head>)/m', '        '.DisplayUtility::microsummary($summary)."\n".'${1}', $source);
                }
                unset($summary);
            }

        } /* end if */

        return $source;
    }

    /**
     * smarty modifier date
     *
     * @access  public
     * @static
     * @param   string  $time
     * @return  string
     */
    function date($time)
    {
        /* settype to INTEGER */
        $time = (int) $time;
        global $YANA;

        if (empty($time)) {
            $time = time();
        }

        $profileTimeFormat = $YANA->getVar("PROFILE.TIMEFORMAT");
        if (!is_numeric($profileTimeFormat)) {
            $profileTimeFormat = 0;
        }
        $timeformat = $YANA->getVar("DATE.".$profileTimeFormat);

        $script = "";
        $ref = array('LANGUAGE' => $YANA->getVar('LANGUAGE'));

        /* provide javascript */
        if ((is_array($timeformat) && isset($timeformat['JS'])) || is_string($timeformat)) {
            $script .= '<script type="text/javascript" language="JavaScript">';
            $script .= 'date=new Date('.$time."000);document.write(";
            if (is_string($timeformat)) {
                $script .= Layout::fillTemplate($timeformat, $ref);
            } else {
                $script .= Layout::fillTemplate($timeformat['JS'], $ref);
            }
            $script .= ");</script>";
        }
        /* provide textual representation for fall back */
        if (is_array($timeformat) && isset($timeformat['PHP'])) {
            $timeformat['PHP'] = Layout::fillTemplate($timeformat['PHP'], $ref);
            $script .= '<noscript>'.date($timeformat['PHP'],$time).'</noscript>';
        }
        return $script;

    }

    /**
     * smarty modifier entities
     *
     * @access  public
     * @static
     * @param   string  $string
     * @return  string
     */
    function entities($string)
    {
        return htmlentities($string);
    }

    /**
     * smarty modifier url
     *
     * @access  public
     * @static
     * @param   string   $string           url parameter list
     * @param   boolean  $asString         decide wether entities in string should be encoded or not
     * @param   boolean  $asAbsolutePath   decide wether function should return relative or absolut path
     * @return  string
     */
    function url($string, $asString = false, $asAbsolutePath = true)
    {

        /* settype to STRING */
        $string = (string) $string;
        global $YANA;

        preg_match_all("/(&|^)(.*)=(.*)(&|$)/U", $string, $m);
        for ($i = 0; $i < count($m[0]); $i++)
        {
            $string = preg_replace("/".toRegExp($m[0][$i])."/", $m[1][$i].urlencode($m[2][$i])."=".urlencode($m[3][$i]).$m[4][$i], $string);
        }

        $url = "";

        if ($asAbsolutePath === true) {
            if (isset($_SERVER['HTTPS']) && strcasecmp($_SERVER['HTTPS'], "on") === 0) {
                $url = "https://";
            } else {
                $url = "http://";
            }
            if (isset($_SERVER['HTTP_HOST'])) {
                $url .= $_SERVER['HTTP_HOST'];
            } else {
                $url .= $_SERVER['SERVER_NAME'];
            }
            $url .= dirname($_SERVER['PHP_SELF']). '/';
        }
        if (isset($YANA)) {
            $url .= $YANA->getVar('PHP_SELF')."?".((@session_id() != "") ? @session_name()."=".@session_id() : "");
            if ($asString === true) {
                $url .= "&id=".$YANA->getVar('ID')."&".$string;
            } else {
                $url .= "&amp;id=".$YANA->getVar('ID')."&amp;".htmlentities($string);
            }
        } else {
            $url .= $_SERVER['PHP_SELF']."?".((@session_id() != "") ? @session_name()."=".@session_id() : "");
        }
        return $url;

    }

    /**
     * smarty modifier href
     *
     * @access  public
     * @static
     * @param   string  $time
     * @return  string
     */
    function href($string)
    {

        return '"'.DisplayUtility::url($string, false, false).'"';

    }

    /**
     * smarty modifier microsummary
     *
     * @access  public
     * @static
     * @param   string  $target
     * @return  string
     */
    function microsummary($target)
    {
        /* settype to STRING */
        $url = (string) $url;
        if (!preg_match("/^\w[\w\d-_]*$/si", $target)) {
            return "";
        } else {
            return '<link rel="microsummary" href="'.$_SERVER['PHP_SELF']."?action=get_microsummary&amp;target=$target".'" type="text/plain">';
        }
    }

    /**
     * smarty modifier css
     *
     * @access  public
     * @static
     * @param   string  $url
     * @return  string
     */
    function css($url)
    {
        /* settype to STRING */
        $url = (string) $url;
        if (!preg_match("/^[\w-_\.\/]+\.css$/si", $url)) {
            return "";
        } else {
            return '<link rel="stylesheet" type="text/css" href="'.$url.'">';
        }
    }

    /**
     * smarty modifier script
     *
     * @access  public
     * @static
     * @param   string  $time
     * @return  string
     */
    function script($string)
    {
        /* settype to STRING */
        $string = (string) $string;
        if (!preg_match("/^[\w-_\.\/]+\.js$/si", $string)) {
            return "";
        } else {
            return '<script type="text/javascript" language="javascript" src="'.$string.'"></script>';
        }
    }

    /**
     * smarty modifier embeddedTags
     *
     * @access  public
     * @static
     * @param   string  $source
     * @param   bool    $use_html_entities
     * @return  string
     */
    function embeddedTags($txt, $use_html_entities = true)
    {
        if (!is_string($txt)) {
            return $txt;
        }

        if (!is_bool($use_html_entities)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, '2 in '.__FUNCTION__.'()', 'Boolean', gettype($use_html_entities)), E_USER_ERROR);
            return $txt;
        }

        /* if not necessary -> skip the whole section for better performance */
        if (is_int(strpos($txt, '['))) {

            $invalid_resource = '<img title="the resource %RESOURCE% was blocked because it contained illegal characters" alt="invalid %RESOURCE%" border="0" src="%DATADIR%icon_x.gif">';

            $txt = preg_replace("/\[php\](.*)(\[\/php\]|$)/Usi", "[code]<?php\n\\1\n?>[/code]", $txt);
            while (preg_match('/\[code\](.*)(\[\/code\]|$)/Us', $txt, $m))
            {
                $m[1] = preg_replace("/\[\/?(b|i|emp|h|code|php|img|mark|color|url|mail)\]/si", "", $m[1]);
                /* This is to avoid double quoting , since highlight_string() will quote the string again. */
                $m[1] = html_entity_decode($m[1]);
                $m[1] = '<code style="white-space: pre-wrap">'.highlight_string($m[1],true).'</code>';
                $txt  = str_replace($m[0], $m[1], $txt);
            }
            unset($m);
            $txt = preg_replace("/\[b\](.*)(\[\/b\]|$)/Us",               "<span style=\"font-weight: bold;\">\\1</span>", $txt);
            $txt = preg_replace("/\[i\](.*)(\[\/i\]|$)/Us",               "<span style=\"font-style: italic;\">\\1</span>", $txt);
            $txt = preg_replace("/\[u\](.*)(\[\/u\]|$)/Us",               "<span style=\"text-decoration: underline;\">\\1</span>", $txt);
            $txt = preg_replace("/\[emp\](.*)(\[\/emp\]|$)/Us",           "<span class=\"EMPHASIZE\">\\1</span>", $txt);
            $txt = preg_replace("/\[h\](.*)(\[\/h\]|$)/Us",               "<p class=\"HIGHLIGHT\">\\1</p>", $txt);
            $txt = preg_replace("/\[c\](.*)(\[\/c\]|$)/Us",               "<div class=\"KOMMENTAR\">\\1</div>", $txt);
            $txt = preg_replace("/\[hide\](.*)(\[\/hide\]|$)/Us",         "<div style=\"padding: 5px; border: 1px solid #555555; background-color: #CCCCCC; color: #CCCCCC\" onMouseOver=\"this.style.backgroundColor='#F0F0F0'; this.style.color='#333333'\" onMouseOut=\"this.style.backgroundColor='#CCCCCC'; this.style.color='#CCCCCC'\">\\1</div>", $txt);

            $txt = preg_replace("/\[small\](.*)(\[\/small\]|$)/Us",       "<span style=\"font-size: x-small\">\\1</span>", $txt);
            $txt = preg_replace("/\[big\](.*)(\[\/big\]|$)/Us",           "<span style=\"font-size: large\">\\1</span>", $txt);
            $txt = preg_replace("/\[mark\](.*)(\[\/mark\]|$)/Usi",         "<span style=\"background-color:#FFFF00\">\\1</span>", $txt);
            $txt = preg_replace("/\[mark=(\w+|\#[\da-fA-F]{3}|\#[\da-fA-F]{6})\](.*)(\[\/mark\]|$)/Us", '<span style="background-color:${1}">${2}</span>', $txt);
            $txt = preg_replace("/\[color\](.*)(\[\/color\]|$)/Us",       "<span style=\"color:#000000\">\\1</span>", $txt);
            $txt = preg_replace("/\[color=(\w+|\#[\da-fA-F]{3}|\#[\da-fA-F]{6})\](.*)(\[\/color\]|$)/Us", '<span style="color:${1}">${2}</span>', $txt);

            $txt = preg_replace("/\[mail=mailto:(.*)\]/Us",               "[mail=\\1]", $txt);
            $txt = preg_replace("/\[mail]mailto:(.*)\[\/mail\]/Us",       "[mail]\\1[/mail]", $txt);

            /* images */
            $matches = array();
            while (preg_match("/\[img\](.*)\[\/img\]/U", $txt, $matches))
            {
                if (preg_match("/^[\w\d-_\/]+\.(png|jpg|gif|jpeg)$/i", $matches[1], $ext)) {
                    $txt = str_replace($matches[0], '<img alt="" border="0" src="'.htmlentities(strip_tags(preg_replace("/\[wbr\]/i", "", $matches[1]))).'" style="max-width: 320px; max-height: 240px" onload="javascript:if(this.width>320) { this.width=320; }; if(this.height>240) { this.height=240; };">', $txt);
                } else {
                    $txt = str_replace($matches[0], str_replace('%RESOURCE%', 'image', $invalid_resource), $txt);
                }
            } /* end foreach */
            $matches = array();

            /* mail */
            /* may contain word-/line- breaks that need to be removed */
            while (preg_match('/\[mail=[^\[\]]*\[wbr\]/i', $txt))
            {
                $txt   = preg_replace('/(\[mail=[^\[\]]*)\[wbr\]/i', '${1}', $txt);
            }
            unset($count);
            while (preg_match("/\[mail=(.*)\](.*)\[\/mail\]/Usi", $txt, $matches1) || preg_match("/\[mail\](.*)\[\/mail\]/Ui", $txt, $matches2))
            {
                if (!empty($matches1)) {
                    $mail_match =& $matches1[0];
                    $mail_href  =  $matches1[1];
                    $mail_text  =  $matches1[2];
                } else if (!empty($matches2)) {
                    $mail_match =& $matches2[0];
                    $mail_href  =  $matches2[1];
                    $mail_text  =  $matches2[1];
                    $mail_text  =  preg_replace('/ ?(\[wbr\]|\[br\])/','',$mail_text);
                    if (strlen($mail_text) > 40) {
                        $mail_text = substr($mail_text,0,34) . "..." . substr($mail_text,-3);
                    }
                }
                /* Count in the possibility of some nasty guy injecting white space characters */
                $mail_href = preg_replace('/[\x00-\x1f]/','',$mail_href);
                /* Although there should not be any tags - you never know ... */
                $mail_href = strip_tags($mail_href);
                $mail_href = preg_replace('/ ?(\[wbr\]|\[br\])/','',$mail_href);
                $mail_href = preg_replace('/^mailto:/i', '', $mail_href);
                $mail_href = untaintInput($mail_href, 'mail');
                $mail_href = htmlentities($mail_href);
                if (!empty($mail_href)) {
                    $txt = str_replace($mail_match, '<a href="&#109;&#97;&#105;&#108;&#116;&#111;&#58;'.$mail_href.'" target="_blank">'.$mail_text.'</a>', $txt);
                } else {
                    $txt = str_replace($mail_match, str_replace('%RESOURCE%', 'mail', $invalid_resource), $txt);
                }
            } /* end while */
            $matches1 = array();
            $matches2 = array();

            /* url */
            /* may contain word-/line- breaks that need to be removed */
            while (preg_match('/\[url=[^\[\]]*(?:\[wbr\]|\[br\])/si', $txt))
            {
                $txt   = preg_replace('/(\[url=[^\[\]]*)(?:\[wbr\]|\[br\])/si', '${1}', $txt);
            }
            unset($count);
            while (preg_match("/\[url=(.*)\](.*)\[\/url\]/Usi", $txt, $matches1) || preg_match("/\[url\](.*)\[\/url\]/Ui", $txt, $matches2))
            {
                if (!empty($matches1)) {
                    $uri_match =& $matches1[0];
                    $uri_href  =  $matches1[1];
                    $uri_text  =  $matches1[2];
                } else if (!empty($matches2)) {
                    $uri_match =& $matches2[0];
                    $uri_href  =  $matches2[1];
                    $uri_text  =  $matches2[1];
                    $uri_text  =  preg_replace('/ ?(\[wbr\])/','',$uri_text);
                    if (strlen($uri_text) > 40) {
                        $uri_text = substr($uri_text,0,34) . "..." . substr($uri_text,-3);
                    }
                }
                /* Count in the possibility of some nasty guy injecting white space characters */
                $uri_href = preg_replace('/[\x00-\x1f]/','',$uri_href);
                /* Although there should not be any tags - you never know ... */
                $uri_href = strip_tags($uri_href);
                $uri_href = preg_replace('/ ?(\[wbr\]|\[br\])/','',$uri_href);
                $uri_href = untaintInput($uri_href, 'url');
                $uri_href = htmlentities($uri_href);
                if (!preg_match('/^(https?:\/\/|ftp:\/\/)/', $uri_href) && preg_match('/^[^:]+:/', $uri_href)) {
                    $uri_href = '';
                } else if (!preg_match('/^[^:]+:/', $uri_href)) {
                    $uri_href = 'http://'.$uri_href;
                }
                if (!empty($uri_href)) {
                    $txt = str_replace($uri_match, '<a href="'.$uri_href.'" target="_blank">'.$uri_text.'</a>', $txt);
                } else {
                    $txt = str_replace($uri_match, str_replace('%RESOURCE%', 'uri', $invalid_resource), $txt);
                }
                unset($uri_match);
                unset($uri_href);
                unset($uri_text);
            }
            $matches1 = array();
            $matches2 = array();

            /* line break */
            $txt = str_replace('[br]', '<br>', $txt);

            /* word break */
            $txt = str_replace('[wbr]', '<span style="font-size: 0px"> </span>', $txt);

            /* remove uncaught tags */
            $txt = preg_replace("/\[\/?(b|i|emp|h|code|php|img|mark|color|url|mail)\]/si", "", $txt);
        }

        assert('is_string($txt);');
        return $txt;
    }

    /**
     * smarty modifier scanForAt
     *
     * @access  public
     * @static
     * @param   string  $source
     * @return  string
     */
    function scanForAt($source)
    {

        if (!is_string($source)) {
            return $source;
        } else {
            preg_match_all("/[\w\.\-_]+@[\w\.\-_]+/", $source, $matches);

            for ($i = 0; $i < count($matches[0]); $i++)
            {
                $source = str_replace($matches[0][$i], String::htmlEntities($matches[0][$i]), $source);
            }

            return $source;
        }

    }

    /**
     * smarty modifier smilies
     *
     * @access  public
     * @static
     * @param   string  $txt
     * @return  string
     */
    function smilies($txt)
    {
        assert('empty($txt) || is_string($txt); // Wrong argument type. String expected found '.gettype($txt).' instead.');
        if (!is_string($txt)) {
            return $txt;
        }

        global $YANA;
        if (isset($YANA)) {
            $smilies = $YANA->getVar('SMILIES');
            $smilies_dir = $YANA->getVar('PROFILE.SMILEYDIR');
        } else {
            global $smilies, $smilies_dir;
        }

        /* default smilies */
        if (isset($smilies) && is_array($smilies)) {
            if (in_array("smile", $smilies))        {
                $txt = preg_replace("/:[\-oO]?\)/", '<img alt=":-)" border="0" hspace="2" src="'.$smilies_dir.'smile.gif">', $txt);
            }
            if (in_array("teufelchen", $smilies))   {
                $txt = preg_replace("/>:[\-oO]?\)/", '<img alt="&gt-)" border="0" hspace="2" src="'.$smilies_dir.'teufelchen.gif">', $txt);
            }
            if (in_array("boese", $smilies))        {
                $txt = preg_replace("/:[\-oO]?\(/", '<img alt=":-(" border="0" hspace="2" src="'.$smilies_dir.'boese.gif">', $txt);
            }
            if (in_array("keinemeinung", $smilies)) {
                $txt = preg_replace("/:[\-oO]?\|/", '<img alt=":-|" border="0" hspace="2" src="'.$smilies_dir.'keinemeinung.gif">', $txt);
            }
            if (in_array("lachen", $smilies))       {
                $txt = preg_replace("/:[\-oO]?D/", '<img alt=":-D" border="0" hspace="2" src="'.$smilies_dir.'lachen.gif">', $txt);
            }
            if (in_array("grinsen", $smilies))      {
                $txt = preg_replace("/;[\-oO]?\)/", '<img alt=";-)" border="0" hspace="2" src="'.$smilies_dir.'grinsen.gif">', $txt);
            }
            if (in_array("zunge", $smilies))        {
                $txt = preg_replace("/:[\-oO]?P/", '<img alt=":-P" border="0" hspace="2" src="'.$smilies_dir.'zunge.gif">', $txt);
            }
            if (in_array("traurig", $smilies))      {
                $txt = preg_replace("/:[\'\*]\(/", '<img alt=":*(" border="0" hspace="2" src="'.$smilies_dir.'traurig.gif">', $txt);
            }
        }

        /* if not necessary -> skip the whole section for better performance */
        if (is_int(strpos($txt, ':'))) {
            /* Emot-Codes */
            for ($j = 0; $j < count($smilies); $j++)
            {
                while (preg_match("/:".$smilies[$j].":(\s|\[wbr\]|\[br\]|<br \/>)*:".$smilies[$j].":/i", $txt))
                {
                    $txt = preg_replace("/:".$smilies[$j].":(\s|\[wbr\]|\[br\]|<br \/>)*:".$smilies[$j].":/i", ':'.$smilies[$j].':', $txt);
                }
                $txt = preg_replace("/:".addcslashes($smilies[$j], "+()[]{}.?*/\\$^").":/", '<img alt="" border="0" hspace="2" src="'.$smilies_dir.$smilies[$j].'.gif">', $txt);
            }
        }

        assert('is_string($txt);');
        return $txt;
    }

    /**
     * smarty function printArray
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function printArray($params, &$smarty)
    {
        if (!isset($params['value'])) {
            return "";
        } else {
            $array = $params['value'];
        }
        if (is_string($array)) {
            $array = SML::decode($array);
        }
        if (is_array($array)) {
            return '<pre>'.htmlentities(SML::encode($array)).'</pre>';
        } else {
            return $array;
        }
    }

    /**
     * smarty function print unordered list
     *
     * Print an array using a tree menu.
     *
     * The following CSS classes are used:
     * <ul>
     *  <li> ul.gui_array_list </li>
     *  <li> ul.gui_array_list &gt; li.gui_array_list </li>
     *  <li> ul.gui_array_list &gt; li.gui_array_head </li>
     *  <li> ul.gui_array_list &gt; li.gui_array_list &gt; span.gui_array_key </li>
     *  <li> ul.gui_array_list &gt; li.gui_array_list &gt; span.gui_array_value </li>
     *  <li> ul.gui_array_list &gt; li.gui_array_head &gt; span.gui_array_key </li>
     *  <li> ul.gui_array_list &gt; li.gui_array_head &gt; span.gui_array_value </li>
     * </ul>
     *
     * If you set the argument 'keys_as_href' to true,
     * the tree menu will create links with the array
     * values as link text and the array keys as hrefs.
     * Links are only created if the value is scalar,
     * which means you may create sub-menues by using
     * another array as the value.
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function printUnorderedList($params, &$smarty)
    {
        if (!isset($params['value'])) {
            return "";
        } else {
            $array = $params['value'];
        }
        if (empty($params['keys_as_href'])) {
            $keysAsHref = false;
        } else {
            $keysAsHref = true;
        }
        if (is_string($array)) {
            $array = SML::decode($array);
        }
        if (is_array($array)) {
            return DisplayUtility::_printUnorderedList($array, $keysAsHref);
        } else {
            return $array;
        }
    }

    /**
     * print unordered list
     *
     * called by DisplayUtility::printUnorderedList()
     *
     * @access  private
     * @static
     * @param   array   $array
     * @param   bool    $keysAsHref
     * @return  string
     * @ignore
     */
    function _printUnorderedList($array, $keysAsHref = false)
    {
        $ul = '<ul class="gui_array_list">';
        foreach ($array as $key => $element)
        {
            /* print key */
            if (is_array($element)) {
                $ul .= '<li class="gui_array_head" onmouseover="this.className=\'gui_array_head_open\'" onmouseout="this.className=\'gui_array_head\'">';
                $ul .= '<span class="gui_array_key">';
                $ul .= htmlentities($key);
                $ul .= '</span>';
            } else {
                $ul .= '<li class="gui_array_list">';
                if ($keysAsHref && is_scalar($element)) {
                    $ul .= '<a href="'.htmlentities($key).'">';
                } else {
                    $ul .= '<span class="gui_array_key">';
                    $ul .= htmlentities($key).':';
                    $ul .= '</span>';
                }
            }

            /* print value */
            if (is_bool($element)) {
                $ul .= '<span class="gui_array_value">';
                if ($element) {
                    $ul .= '<img alt="true" title="true" src="%DATADIR%boolean_true.gif">';
                } else {
                    $ul .= '<img alt="false" title="false" src="%DATADIR%boolean_false.gif">';
                }
                $ul .= '</span>';
            } else if (is_scalar($element)) {
                $ul .= '<span class="gui_array_value">'.htmlentities($element).'</span>';
            } else if (is_array($element)) {
                $ul .= DisplayUtility::_printUnorderedList($element, $keysAsHref);
            } else if (is_object($element)) {
                if (is_subclass_of($element, 'Object')) {
                    $ul .= '<span class="gui_array_value">'.htmlentities($element->toString()).'</span>';
                } else {
                    $ul .= '<span class="gui_array_value">'.'Instance of '.getclass($element).'</span>';
                }
            } else {
                $ul .= '<span class="gui_array_value">'.htmlentities(print_r($element, true)).'</span>';
            }

            /* close open 'a' tag */
            if ($keysAsHref && is_scalar($element)) {
                $ul .= '</a>';
            }
            $ul .= '</li>';
        }
        $ul .= '</ul>';
        return $ul;
    }

    /**
     * smarty function loop through elements of an array
     *
     * This is pretty much the same as Smarty's "foreach" block-function,
     * excepts for the fact that it flattens multidimensional arrays
     * to one-dimension by including the '.' seperator to create compound
     * keys.
     *
     * Example:
     * <code>
     * $array = array('a' = 1, 'b' => array('foo', 'bar'), 'c' = 2);
     *
     * [%foreach from=$array item="key" key="element"%]
     * [%$key%] = [%$element%]
     * [%/foreach%]
     *
     * the code above will output:
     * a = 1
     * b = Array
     * c = 2
     *
     * Note that 'foreach' converts $element to a string,
     * even if it is was an array.
     *
     * [%loop from=$array item="key" key="element"%]
     * $key = $element
     * [%/loop%]
     *
     * the code above will output:
     * a = 1
     * b.0 = foo
     * b.1 = bar
     * c = 2
     *
     * Note that 'loop' does not convert $element to a string
     * if it is an array, but provides a list of it's items instead.
     *
     * You should also note the difference in the writing of the token.
     * While 'foreach' writes [%$key%] the function 'loop' uses just $key.
     * </code>
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   string  $source
     * @param   Smarty  &$smarty
     * @param   mixed   &$repeat
     * @return  string
     * @ignore
     */
    function loopArray($params, $template, &$smarty, &$repeat)
    {
        if (!is_array($params['from'])) {
            return "";
        }

        if (!$repeat) {
            return DisplayUtility::_loopArray(@$params['key'], @$params['item'], $params['from'], $template);
        }
    }

    /**
     * loop through array
     *
     * called by DisplayUtility::loopArray()
     *
     * @access  private
     * @static
     * @param   string  $id
     * @param   string  $name
     * @param   array   &$array
     * @param   string  $prefix
     * @return  string
     * @ignore
     */
    function _loopArray($key, $item, &$array, &$template, $prefix = "")
    {
        $list = '';
        assert('!isset($id);      // Cannot redeclare $id');
        assert('!isset($element); // Cannot redeclare $element');
        foreach ($array as $id => $element)
        {
            if (is_array($element)) {
                $list .= DisplayUtility::_loopArray($key, $item, $element, $template, $prefix.$id.'.');
            } else if (is_scalar($element)) {
                $li = $template;
                $li = str_replace('$'.$key, htmlentities($prefix.$id), $li);
                $li = str_replace('$'.$item, htmlentities($element), $li);
                $list .= $li;
            }

        } /* end foreach */
        unset($id, $element);
        return $list;
    }

    /**
     * smarty function guiEmbeddedTags
     *
     * @static
     * @access  public
     * @name    DisplayUtility::guiEmbeddedTags()
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     *
     * @see     DisplayUtility::loadSmilies()
     */
    function guiEmbeddedTags($params, &$smarty)
    {
        global $YANA;

        $list_of_tags = array('b','i','u','h','emp','c','small','big','hide','code','img','url','mail','color','mark');

        /* Argument 'show' */
        if (isset($params['show']) && !is_string($params['show'])) {
            $smarty->trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'show in '.__FUNCTION__.'()', 'String', gettype($params['show'])), E_USER_WARNING);
            return "";
        } else if (isset($params['show']) && !preg_match('/^(\w+|\||-)(,(\w+|\||-))*$/is',$params['show'])) {
            $smarty->trigger_error("Argument 'show' contains illegal characters for function ".__FUNCTION__."().", E_USER_WARNING);
            return "";
        } else if (!isset($params['show'])) {
            $show =& $list_of_tags;
        } else {
            $show = explode(',', strtolower($params['show']));
        }

        /* Argument 'hide' */
        if (!empty($params['hide']) && !is_string($params['hide'])) {
            $smarty->trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'hide in '.__FUNCTION__.'()', 'String', gettype($params['hide'])), E_USER_WARNING);
            return "";
        } else if (!empty($params['hide']) && !preg_match('/^[\w,]+$/is',$params['hide'])) {
            $smarty->trigger_error("Argument 'hide' contains illegal characters for function ".__FUNCTION__."().", E_USER_WARNING);
            return "";
        } else if (empty($params['hide'])) {
            $hide = array();
        } else {
            $hide = explode(',', strtolower($params['hide']));
        }

        assert('is_array($show);');
        assert('is_array($hide);');

        $tags = array_diff($show, $hide);

        /* create document */
        $document = new smartTemplate();
        $document->insert('TAGS', $tags);
        $document->insert('EMB_TAGS_ID', $target);
        $document->insert('LANGUAGE', $YANA->getVar('LANGUAGE'));
        $document->setPath($YANA->skin->getFile("gui_embedded_tags"));

        return $document->get();
    }

    /**
     * smarty function guiSmlLoad
     *
     * The function sml_load() provides the same interface as
     * the built-in smarty function config_load().
     * The difference is, that sml_load() works on SML files.
     *
     * In addition you may use the scope 'template_vars', which
     * is unique for sml_load(). This scope merges the values
     * with the current array of template vars instead of copying
     * them to the config vars of the template. This enables you
     * to handle loaded SML structure as common vars and provides
     * access to nested structures inside SML files, which is
     * a feature that is not available for normal smarty config files.
     *
     * @static
     * @access  public
     * @name    DisplayUtility::guiSmlLoad()
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function guiSmlLoad($params, &$smarty)
    {

        /* input checking*/

        if (empty($params['file'])) {
            $smarty->trigger_error("Missing argument 'file' in function sml_load()", E_USER_WARNING);
            return "";
        } else if (!is_string($params['file'])) {
            $smarty->trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'file in sml_load()', 'String', gettype($params['file'])), E_USER_WARNING);
            return "";
        } else if (!preg_match('/^[\w\d-_]+\.(?:sml|config)$/i', $params['file'])) {
            $smarty->trigger_error("Argument 'file' contains illegal characters for function sml_load().", E_USER_WARNING);
            return "";
        } else if (!is_file($smarty->config_dir.$params['file'])) {
            $smarty->trigger_error("Argument 'file' is not a valid file path in function sml_load().", E_USER_WARNING);
            return "";
        } else {
            $in_file = $smarty->config_dir.$params['file'];
        } /* end if */

        if (empty($params['section']) || $params['section'] === '*') {
            $in_section = null;
        } else if (!is_string($params['section'])) {
            $smarty->trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'section in sml_load()', 'String', gettype($params['section'])), E_USER_WARNING);
            return "";
        } else if (!preg_match('/^([\w\d-_\.]+)$/i', $params['section'])) {
            $smarty->trigger_error("Argument 'section' contains illegal characters for function sml_load().", E_USER_WARNING);
            return "";
        } else {
            $in_section = $params['section'];
        } /* end if */

        if (empty($params['scope'])) {
            if (empty($params['global'])) {
                $in_scope = 'local';
            } else {
                if (!is_boolean($params['global'])) {
                    $smarty->trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'global in sml_load()', 'Boolean', gettype($params['global'])), E_USER_WARNING);
                    return "";
                } else {
                    $smarty->trigger_error("Argument 'global' is deprecated in function sml_load(). You should consider using scope='parent' (equals global=true) or scope='local' (equals global=false) instead.", E_USER_NOTICE);
                    if ($params['global'] === true) {
                        $in_scope = 'parent';
                    } else if ($params['global'] === false) {
                        $in_scope = 'local';
                    } else {
                        $in_scope = 'local';
                    } /* end if */
                } /* end if */
            } /* end if */
        } else if (!is_string($params['scope'])) {
            $smarty->trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'scope in sml_load()', 'String', gettype($params['scope'])), E_USER_WARNING);
            return "";
        } else {
            switch(strtolower($params['scope'])) {
                case 'global':
                    $in_scope = 'global';
                break;

                case 'local':
                    $in_scope = 'local';
                break;

                case 'parent':
                    $in_scope = 'parent';
                break;

                case 'template_vars':
                    $in_scope = 'template_vars';
                break;

                default:
                    $smarty->trigger_error("Argument 'scope' has an illegal value for function sml_load(). Accepted values are: 'template_vars', 'global', 'local', 'parent'.", E_USER_WARNING);
                    return "";
                break;
            } /* end switch */
        } /* end if */

        unset($params);

        /* cache handling */
        $cacheFile = $smarty->compile_dir.'sml_load'.md5($in_file).'.php';
        if ($smarty->force_compile == true || !file_exists($cacheFile) || filemtime($in_file) > filemtime($cacheFile)) {
            $forceCompile = true;
        } else {
            $forceCompile = false;
        }
        clearstatcache();

        /* business logic */
        if ($forceCompile === true) {
            assert('!isset($array);');
            $array = SML::getFile($in_file, CASE_UPPER);
            $cache = new SecureFileStream($cacheFile);
            $cache->read();
            if (!$cache->exists()) {
                $cache->create();
            }
            $cache->insert('<?php $array = unserialize(\''.serialize($array).'\'); ?>');
            $cache->write();
            unset($cache);
        } else {
            include($cacheFile);
        }
        if (!empty($in_section)) {
            assert('is_array($array); /* unexpected result: $array should be an array */');
            $array = Hashtable::get($array, $in_section);
        }

        /* resolve scope argument */
        if ($in_scope == 'template_vars') {
            $i_max = -1;
        } else if ($in_scope == 'local') {
            $i_max = 0;
        } else if ($in_scope == 'parent') {
            $i_max = 1;
        } else if ($in_scope == 'global') {
            $i_max = count($smarty->_config);
        } else {
            $i_max = UNDEFINED;
            $smarty->_config_overwrite(null);
            return "";
        }

        /* copy values to smarty config vars */
        if ($in_scope == 'template_vars') {
            $keys = array_keys($array);
            for ($i = 0; $i < count($keys); $i++)
            {
                $smarty->assign_by_ref($keys[$i], $array[$keys[$i]], true);
            }
        } else {
            for ($i = 0; $i <= $i_max; $i++)
            {
                if (!isset($smarty->_config[$i])) {
                    $smarty->_config[$i] = array('vars' => array(), 'files' => array());
                }
                if (empty($smarty->_config[$i]['vars'])) {
                    $smarty->_config[$i]['vars'] = $array;
                } else {
                    $smarty->_config[$i]['vars'] = Hashtable::merge($smarty->_config[$i]['vars'], $array);
                }
                $smarty->_config[$i]['files'][$in_file] = true;

            }
        }

        return "";
    }

    /**
     * smarty function guiSmilies
     *
     * @static
     * @access  public
     * @name    DisplayUtility::guiSmilies()
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     *
     * @see     DisplayUtility::loadSmilies()
     */
    function guiSmilies($params, &$smarty)
    {
        global $YANA;
        $TABLE_smilies   = '<table summary="smilies" class="gui_generator_smilies"><tr>';
        $title_smilies   = $YANA->getVar("LANGUAGE.TITLE_SMILIES");
        $smileyDir       = $YANA->getVar("PROFILE.SMILEYDIR");
        $smilies         = $YANA->getVar("SMILIES");
        $params['width'] = (int) $params['width'];
        if ($params['width'] < 1) {
            $params['width'] = 1;
        }

        for ($j = 0; $j < count($smilies); $j++)
        {
            if ($j % $params['width'] == 0 && $j > 0) {
                $TABLE_smilies .= '</tr><tr>';
            }
            $TABLE_smilies .= '<td><a title="'.$title_smilies.'" href="javascript://:'.urlencode($smilies[$j]).':"><img alt="'.$smilies[$j].'" src="'.$smileyDir.$smilies[$j].'.gif" onmousedown="addIcon(\':'.$smilies[$j].':\',event)"></a></td>'."\n";
        }

        return $TABLE_smilies."</tr></table>";
    }

    /**
     * loadSmilies
     *
     * @static
     * @access  public
     * @name    DisplayUtility::loadSmilies()
     * @param   string  $user_dir (optional)
     *
     * @see     DisplayUtility::guiSmilies()
     */
    function loadSmilies($user_dir = "./")
    {
        assert('is_dir($user_dir); /* The value \''.$user_dir.'\' is not a directory */');
        if (!is_string($user_dir)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($user_dir)), E_USER_WARNING);
        } else {
            global $YANA;
            if (isset($YANA)) {
                $smilies_dir = $YANA->getVar('PROFILE.SMILEYDIR');
            } else {
                global $smilies, $smilies_dir;
                $smilies_dir = $user_dir;
            }

            if (is_dir($smilies_dir)) {
                if (empty($smilies) || is_null($smilies)) {
                    $smilies = array();

                    $dir = dir($smilies_dir);
                    while ($txt = $dir -> read())
                    {
                        if (preg_match("/\.gif$/i", $txt)) {
                            $smilies[count($smilies)] = substr($txt, 0, strrpos($txt, "."));
                        }
                    }
                    $dir->close();
                    sort($smilies);
                }

                if (isset($YANA)) {
                    $YANA->setVar('SMILIES', $smilies);
                }
            } else {
                trigger_error("Unable to load smilies. The directory '${smilies_dir}' does not exist.",E_USER_WARNING);
            }
        } /* end if */
    }

    /**
     * smarty function guiImport
     *
     * @static
     * @access  public
     * @param   array   $param
     * @param   Smarty   &$smarty
     * @return  string
     */
    function guiImport($param, &$smarty)
    {
        if (isset($param['file'])) {

            $filename = $param['file'];
            /* input check */
            if (!is_string($filename)) {
                trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'file', 'String', gettype($filename)), E_USER_WARNING);
                return false;
            } else if (!is_file($filename)) {
                trigger_error("Invalid argument 'file'. The file '${filename}' does not exist.", E_USER_WARNING);
                return false;
            }

        } else if (isset($param['id'])) {
            
            /* input check */
            if (!is_string($param['id'])) {
                trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 'id', 'String', gettype($param['id'])), E_USER_WARNING);
                return false;
            } else {
                $id = $param['id'];
            }

            global $YANA;
            $filename = $YANA->skin->getFile($id);
            
            if (!is_file($filename)) {
                trigger_error("Invalid argument 'id'. There is not file associated to the id '${id}'.", E_USER_WARNING);
                return false;
            }

        } else {

            trigger_error("Missing argument. You need to provide either the argument 'file' or 'id'.", E_USER_WARNING);
            return false;

        }

        $document = new smartTemplate($filename);
        unset($param['file']);
        if (count($param)>0) {
            $document->insertByReference('*',$param);
        }
        $document->insert('FILE_IS_INCLUDE', "true");
        $include = $document->get();
        return $include;
    }

    /**
     * create a token with quoted delimiters
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _regExpToken($input)
    {
        assert('is_string($input);');
        /* settype to STRING */
        $input = (string) $input;
        return preg_quote(YANA_LEFT_DELIMITER).$input.preg_quote(YANA_RIGHT_DELIMITER);
    }

    /**
     * create a token refering to a variable with quoted delimiters
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _regExpVarToken($input)
    {
        assert('is_string($input);');
        /* settype to STRING */
        $input = (string) $input;
        return preg_quote(YANA_LEFT_DELIMITER).'\$'.strtoupper($input).preg_quote(YANA_RIGHT_DELIMITER);
    }

    /**
     * create a token refering to a variable
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _varToken($input)
    {
        assert('is_string($input);');
        /* settype to STRING */
        $input = (string) $input;
        return YANA_LEFT_DELIMITER.'$'.strtoupper($input).YANA_RIGHT_DELIMITER;
    }

    /**
     * create a token
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _token($input)
    {
        assert('is_string($input);');
        /* settype to STRING */
        $input = (string) $input;
        return YANA_LEFT_DELIMITER.strtoupper($input).YANA_RIGHT_DELIMITER;
    }

}
?>
