<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * @ignore
 */

/**
 * FloodFile
 *
 * This file is intended to be used as a counter.
 *
 * @access      public
 * @package     yana
 * @subpackage  io
 *
 * @ignore
 */
class FloodFile extends SecureFileStream
{

    /**
     * @ignore
     * @access  private
     * @var     int */
    var $max = 0;

    /**
     * add entry to file
     *
     * increment counter and save changes
     *
     * @access  public
     * @return  bool
     */
    function set()
    {
        if (!$this->exists()) {
            $this->create();
        }
        global $YANA;
        if (isset($YANA)) {
            $REMOTE_ADDR = $YANA->getVar('REMOTE_ADDR');
        } else {
            $REMOTE_ADDR = $_SERVER['REMOTE_ADDR'];
        }

        if (!$this->isEmpty() && $REMOTE_ADDR != preg_replace("/\s/", "", $this->content[0]) || ($this->content[1] +20000) < time()) {
            $this->content[0] = $REMOTE_ADDR ."\n";
            $this->content[1] = time() ."\n";
            $this->content[2] = 1;
        } else {
            $this->content[1] = time() ."\n";
            $this->content[2] += 1;
        }
        $test = $this->write();
        assert('is_bool($test);');
        return $test;
    }

    /**
     * alias of set()
     *
     * @access  public
     */
    function insert()
    {
        $this->set();
    }

    /**
     * check if user is blocked
     *
     * Returns bool(true) if the maximum number of entries
     * for this user have been exceeded and bool(false)
     * otherwise.
     *
     * @access  public
     * @return  bool
     */
    function isBlocked()
    {
        $this->read();
        if (!$this->isEmpty() && $this->length()>2) {
            return ($this->content[2] >= $this->max && $this->max > 0);
        } else {
            return false;
        }
    }

    /**
     * set maxmimum number of entries
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   int  $max  a positive number
     * @return  bool
     */
    function setMax($max)
    {
        /**
         * Input checking
         */
        assert('!isset($_chk);');
        $_chk = checkArgumentList(func_get_args(), IS_INT, __CLASS__.'->'.__FUNCTION__);
        if ($_chk !== true) {
            trigger_error($_chk, E_USER_WARNING);
            return false;
        } else {
            unset($_chk);
        }

        if ($max >= 0) {
            $this->max = $max;
            return true;
        } else {
            trigger_error("Expected maximum number to be a positive integer, found '${max}' instead.", E_USER_NOTICE);
            return false;
        }
    }

    /**
     * get maximum number of entries
     *
     * @access  public
     * @return  int
     */
    function getMax()
    {
        assert('is_int($this->max);');
        return $this->max;
    }

}

?>