<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**#@+
 * Yana - Constant
 * @ignore
 */
if (!defined('GUI_OVERVIEW'))   define('GUI_OVERVIEW',   1);
if (!defined('GUI_SEPERATED'))  define('GUI_SEPERATED',  2);
if (!defined('GUI_DETAILS'))    define('GUI_DETAILS',    3);
if (!defined('GUI_EDIT'))       define('GUI_EDIT',       4);
if (!defined('GUI_NEW'))        define('GUI_NEW',        5);
if (!defined('GUI_SEARCH'))     define('GUI_SEARCH',     6);
if (!defined('GUI_ADV_SEARCH')) define('GUI_ADV_SEARCH', 7);
/**#@-*/

/**
 * GUI- and form-generator
 *
 * This class is a GUI- and form-generator,
 * that produces HTML-code from database
 * structure files.
 *
 * @access      public
 * @package     yana
 * @subpackage  core
 *
 * @ignore
 */
class GuiCreator extends Object
{

    /**#@+
     * the following information is MANDATORY
     *
     * @access  private
     */

     /** @var SmartTemplate */ var $template        = null;
     /** @var string        */ var $file            = null;
     /** @var string        */ var $table           = null;

    /**#@-*/
    /**#@+
     * the following information is OPTIONAL
     *
     * @access  private
     */

     /** @var string|array  */ var $where          = array();
     /** @var string        */ var $orderBy        = "";
     /** @var bool          */ var $desc           = false;
     /** @var int           */ var $page           = 0;
     /** @var int           */ var $firstPage      = 0;
     /** @var int           */ var $lastPage       = 0;
     /**
      * the value of the primary key
      * of the current page
      * @var mixed          */ var $currentIndex   = null;
     /**
      * number of entries per page
      * @var int            */ var $entries        = 5;

     /**
      * only applies to some templates
      * @var bool           */ var $titles         = true;

     /**
      * only applies to search forms
      * @var bool           */ var $advancedSearch = false;

    /**#@-*/
    /**#@+
     * the following are values for the ACTION attribute
     *
     * @access  private
     */

     /** @var string        */ var $newAction      = null;
     /** @var string        */ var $editAction     = null;
     /** @var string        */ var $deleteAction   = null;
     /** @var string        */ var $searchAction   = null;

    /**#@-*/
    /**#@+
     * the following are STATIC constants
     *
     * @access  private
     * @static
     */

     /** @var array         */ var $fileMap        = null;
     /** @var array         */ var $tmplMap        = null;
    /**#@-*/
    /**#@+
     * the following are INSTANCE-SPECIFIC var
     *
     * @access  private
     */

    /** @var DbStream       */ var $db             = null;
    /** @var StructureFile  */ var $structure      = null;
    /** @var array          */ var $content        = array();

    /**#@-*/

    /**
     * create a new instance
     *
     * This creates and initializes a new
     * instance of this class.
     */
    function GuiCreator()
    {
        /* mapping of constants to filenames */
        if (is_null($this->fileMap)) {
            global $YANA;

            static $fileMap;
            $fileMap = array(
                GUI_OVERVIEW   =>  $YANA->skin->getFile("gui_view"),
                GUI_SEPERATED  =>  $YANA->skin->getFile("gui_view_seperated"),
                GUI_DETAILS    =>  $YANA->skin->getFile("gui_view_details"),
                GUI_EDIT       =>  $YANA->skin->getFile("gui_edit"),
                GUI_NEW        =>  $YANA->skin->getFile("gui_new"),
                GUI_SEARCH     =>  $YANA->skin->getFile("gui_small_search"),
                GUI_ADV_SEARCH =>  $YANA->skin->getFile("gui_search")
            );
            $this->fileMap =& $fileMap;
        } /* end if */

        /* mapping of strings to filenames */
        if (is_null($this->tmplMap)) {
            static $tmplMap;
            $tmplMap = array(
                'view'           =>  &$this->fileMap[GUI_OVERVIEW],
                'view_seperated' =>  &$this->fileMap[GUI_SEPERATED],
                'view_details'   =>  &$this->fileMap[GUI_DETAILS],
                'edit'           =>  &$this->fileMap[GUI_EDIT],
                'new'            =>  &$this->fileMap[GUI_NEW],
                'search'         =>  &$this->fileMap[GUI_SEARCH],
                'adv_search'     =>  &$this->fileMap[GUI_ADV_SEARCH]
            );
            $this->tmplMap =& $tmplMap;
        } /* end if */

        $this->template = new SmartTemplate();
        $this->template->insertByReference('ADVANCED_SEARCH',   $this->advancedSearch);
        $this->template->insertByReference('ON_NEW',            $this->newAction);
        $this->template->insertByReference('ON_EDIT',           $this->editAction);
        $this->template->insertByReference('ON_DELETE',         $this->deleteAction);
        $this->template->insertByReference('ON_SEARCH',         $this->searchAction);
        $this->template->insertByReference('TABLE',             $this->table);
        $this->template->insertByReference('SHOW_FIELD_TITLES', $this->titles);
        $this->template->insertByReference('SORT',              $this->orderBy);
        $this->template->insertByReference('DESC',              $this->desc);
        $this->template->insertByReference('PAGE',              $this->page);
        $this->template->insertByReference('FIRST_PAGE',        $this->firstPage);
        $this->template->insertByReference('LAST_PAGE',         $this->lastPage);
        $this->template->insertByReference('ENTRIES',           $this->entries);
        $this->template->insertByReference('CURRENT_INDEX',     $this->currentIndex);
        $this->template->insertByReference('CONTENT',           $this->content);

    }

    /**
     * select a template for output
     *
     * You may use either a filename, or one of the following constants:
     *
     * <ul>
     *     <li>  GUI_OVERVIEW   </li>
     *     <li>  GUI_SEPERATED  </li>
     *     <li>  GUI_DETAILS    </li>
     *     <li>  GUI_EDIT       </li>
     *     <li>  GUI_NEW        </li>
     *     <li>  GUI_SEARCH     </li>
     *     <li>  GUI_ADV_SEARCH </li>
     * </ul>
     *
     * Returns bool(true) on success and bool(false) on error.
     * Issues an E_USER_WARNING, if the template is unknown.
     *
     * @access  public
     * @param   int|string  $template
     * @return  bool
     */
    function setTemplate($template)
    {
        /* 1) template is constant value */
        if (is_int($template) && isset($this->fileMap[$template])) {
            /* mapping of constant to filename */
            $filename = $this->fileMap[$template];

        /* 2) template is already a filename */
        } else if (is_string($template) && is_file($template)) {
            $filename = $template;

        /* 3) template is an identifier */
        } else if (is_string($template) && isset($this->tmplMap[$template])) {
            $filename = $this->tmplMap[$template];

        /* 4) template is invalid */
        } else {
            /* error: unknown template */
            trigger_error("The template (".gettype($template).") '".print_r($template, true)."' is unknown.", E_USER_WARNING);
            return false;
        }

        /* set attribute */
        if ($this->template->setPath($filename)) {
            /* success */
            return true;
        } else {
            /* error: e.g. file does not exist or is not readable */
            return false;
        }
    }

    /**
     * get the currently selected template file
     *
     * Returns the filename of the template that is currently,
     * selected, or bool(false) if none has been set.
     *
     * @access  public
     * @return  bool(false)|string
     */
    function getTemplate()
    {
        if (!is_null($this->template)) {
            $filename = $this->template->getPath();
            if (is_string($filename)) {
                /* success */
                return $filename;
            } else {
                /* error: e.g. file does not exist or is not readable */
                return false;
            }
        } else {
            /* error: no template has been selected, yet */
            return false;
        }
    }

    /**
     * set source file
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $file
     * @return  bool
     */
    function setFile($file)
    {
        if (is_string($file)) {
            global $YANA;
            $file     = strtolower($file);
            $this->db = $YANA->connect($file);
            if (is_object($this->db)) {
                $this->db->read();
                if ($this->db->exists()) {
                    $this->file = $file;
                    $this->structure =& $this->db->structure;
                    $this->template->insertByReference('STRUCTURE', $this->structure);
                    if (!empty($this->table)) {
                        $this->content = $this->structure->getByReference("TABLES.".$this->table.".CONTENT");
                    }
                    return true;
                } else {
                    trigger_error("Invalid file argument. Database connection failed", E_USER_WARNING);
                    return false;
                }
            } else {
                return false;
            }
        } else {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($file)), E_USER_WARNING);
            return false;
        }
    }

    /**
     * get the currently selected file
     *
     * Returns the lower-cased name of the currently
     * selected file, or bool(false) if none has been
     * selected yet.
     *
     * @access  public
     * @return  bool(false)|string
     */
    function getFile()
    {
        if (is_string($this->file)) {
            return $this->file;
        } else {
            /* error: no file has been selected, yet */
            return false;
        }
    }

    /**
     * set source table
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $table
     * @return  bool
     */
    function setTable($table)
    {
        if (!is_null($this->structure) && !$this->structure->isTable($table)) {
            return false;
        }

        if (is_string($table)) {
            $table       = strtolower($table);
            $this->table = $table;
            if (is_object($this->template) && is_object($this->structure)) {
                $this->content = $this->structure->getByReference("TABLES.${table}.CONTENT");
            }
            return true;
        } else {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($table)), E_USER_WARNING);
            return false;
        }
    }

    /**
     * get the currently selected table
     *
     * Returns the lower-cased name of the currently
     * selected table, or bool(false) if none has been
     * selected yet.
     *
     * @access  public
     * @return  bool(false)|string
     */
    function getTable()
    {
        if (is_string($this->table)) {
            return $this->table;
        } else {
            /* error: no table has been selected, yet */
            return false;
        }
    }

    /**
     * set action for an event
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $name
     * @param   string  $value
     * @return  bool
     */
    function setAction($name, $value)
    {
        /* input checking */
        assert('is_string($name);  // Wrong argument type for argument 1. String expected.');
        assert('is_string($value); // Wrong argument type for argument 2. String expected.');
        $attribute  = strtolower( (string) $name  );
        $action     = strtolower( (string) $value );
        $attribute .= 'Action';

        /* set action to value */
        if (property_exists($this, $attribute)) {
            $this->$attribute = $action;
            return true;
        } else {
            /* error: no such attribute */
            return false;
        }
    }

    /**
     * get action for an event
     *
     * Returns the lower-cased name of the currently
     * selected action, or bool(false) if none has been
     * selected yet.
     *
     * @access  public
     * @return  bool(false)|string
     */
    function getAction($name)
    {
        assert('is_string($name); // Wrong argument type for argument 1. String expected.');
        $attribute  = strtolower( (string) $name );
        $attribute .= 'Action';

        if (isset($this->$attribute) && is_string($this->$attribute)) {
            return $this->$attribute;
        } else {
            /* error: attribute has not been set, yet */
            return false;
        }
    }

    /**
     * set edit action
     *
     * @see  GuiCreator::setAction()
     */
    function setEditAction($action)
    {
        return $this->setAction('edit', $action);
    }

    /**
     * get edit action
     *
     * @see  GuiCreator::getAction()
     */
    function getEditAction()
    {
        return $this->getAction('edit');
    }

    /**
     * set new action
     *
     * @see  GuiCreator::setAction()
     */
    function setNewAction($action)
    {
        return $this->setAction('new', $action);
    }

    /**
     * get new action
     *
     * @see  GuiCreator::getAction()
     */
    function getNewAction()
    {
        return $this->getAction('new');
    }

    /**
     * set delete action
     *
     * @see  GuiCreator::setAction()
     */
    function setDeleteAction($action)
    {
        return $this->setAction('delete', $action);
    }

    /**
     * get delete action
     *
     * @see  GuiCreator::getAction()
     */
    function getDeleteAction()
    {
        return $this->getAction('delete');
    }

    /**
     * set search action
     *
     * @see  GuiCreator::setAction()
     */
    function setSearchAction($action)
    {
        return $this->setAction('search', $action);
    }

    /**
     * get search action
     *
     * @see  GuiCreator::getAction()
     */
    function getSearchAction()
    {
        return $this->getAction('search');
    }

    /**
     * turn titles on / off
     *
     * Triggers wether fieldnames are to be shown (true=yes, false=no).
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   bool  $titles
     * @return  bool
     */
    function enableTitles($titles = true)
    {
        if (is_bool($titles)) {
            $this->titles = $titles;
            return true;
        } else {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Boolean', gettype($titles)), E_USER_WARNING);
            return false;
        }
    }

    /**
     * get state of titles (on / off)
     *
     * Triggers wether fieldnames are to be shown (true=yes, false=no).
     * Returns bool(true) if enabled, or bool(false) if disabled.
     *
     * @access  public
     * @return  bool
     */
    function getTitles()
    {
        if (is_bool($this->titles)) {
            return $this->titles;
        } else {
            /* error: unexpected value */
            return false;
        }
    }

    /**
     * turn advanced search on / off
     *
     * Triggers wether advanced search is to be used (true=yes, false=no).
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   bool  $advancedSearch
     * @return  bool
     */
    function enableAdvancedSearch($advancedSearch = true)
    {
        if (is_bool($advancedSearch)) {
            $this->advancedSearch = $advancedSearch;
            return true;
        } else {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Boolean', gettype($advancedSearch)), E_USER_WARNING);
            return false;
        }
    }

    /**
     * get state of advanced search (on / off)
     *
     * Triggers wether advanced search is to be used (true=yes, false=no).
     * Returns bool(true) if enabled, or bool(false) if disabled.
     *
     * @access  public
     * @return  bool
     */
    function getAdvancedSearch()
    {
        if (is_bool($this->advancedSearch)) {
            return $this->advancedSearch;
        } else {
            /* error: unexpected value */
            return false;
        }
    }

    /**
     * set column to sort the resultset by
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string  $orderBy  column name
     * @param   bool    $desc     sort descending (true=yes, false=no)
     * @return  bool
     */
    function setSort($orderBy, $desc = false)
    {
        if (!is_string($orderBy)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($orderBy)), E_USER_WARNING);
            return false;
        } else if (!is_null($this->structure) && !$this->structure->isColumn($this->table, $orderBy)) {
            return false;
        } else if (!is_bool($desc)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Boolean', gettype($desc)), E_USER_WARNING);
            return false;
        } else {
            $this->orderBy = strtolower($orderBy);
            $this->desc    = $desc;
            return true;
        }
    }

    /**
     * get the name of the column the resultset is ordered by
     *
     * Returns the lower-cased column name, or bool(false)
     * if none has been set yet.
     *
     * @access  public
     * @return  bool(false)|string
     */
    function getSort()
    {
        if (is_string($this->orderBy) && !empty($this->orderBy)) {
            return $this->orderBy;
        } else {
            /* error: unexpected value */
            return false;
        }
    }

    /**
     * check if resultset is sorted in descending order
     *
     * Returns bool(true) if sorted descending and bool(false) otherwise.
     *
     * @access  public
     * @return  bool
     */
    function isDescending()
    {
        if (is_bool($this->desc)) {
            return $this->desc;
        } else {
            /* error: unexpected value */
            return false;
        }
    }

    /**
     * set current page and offset
     *
     * The first page is 0, the second is 1, aso., defaults to 0.
     * The number of entries per page needs to greater 0, defaults to 5.
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   int  $page     number of start page
     * @param   int  $entries  number of entries per page
     * @return  bool
     */
    function setLimits($page = 0, $entries = 5)
    {
        if (!is_numeric($page)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Integer', gettype($page)), E_USER_WARNING);
            return false;
        } else if (!is_numeric($entries)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Integer', gettype($entries)), E_USER_WARNING);
            return false;
        } else {

            /* settype to INTEGER
             *            INTEGER
             */
            $page    = (int) $page;
            $entries = (int) $entries;

            /* default values */
            if ($page < 0) {
                trigger_error("Page number was negative. Will use the default value 0 instead.", E_USER_NOTICE);
                $page = 0;
            }
            if ($entries < 1) {
                trigger_error("Number of entries per page was less than 1. Will use the default value 5 instead.", E_USER_NOTICE);
                $entries = 5;
            }

            /* preselect hitlist template, by the number of entries per page */
            if (is_object($this->template)) {
                if ($entries > 1) {
                    $this->template->insert('HITLIST_TEMPLATE', $this->fileMap[GUI_OVERVIEW]);
                } else {
                    $this->template->insert('HITLIST_TEMPLATE', $this->fileMap[GUI_DETAILS]);
                }
            }

            /* set values */
            $this->page      = $page;
            $this->firstPage = $page + 1;
            $this->entries   = $entries;
            return true;
        }
    }

    /**
     * get the currently selected page
     *
     * Returns the number of the current page, or bool(false)
     * on error.
     *
     * @access  public
     * @return  bool(false)|int
     */
    function getPage()
    {
        if (is_int($this->page)) {
            return $this->page;
        } else {
            /* error: unexpected value */
            return false;
        }
    }

    /**
     * get number of entries to show per page
     *
     * Returns the number of the current page, or bool(false)
     * on error.
     *
     * @access  public
     * @return  bool(false)|int
     */
    function getEntriesPerPage()
    {
        if (is_int($this->entries)) {
            return $this->entries;
        } else {
            /* error: unexpected value */
            return false;
        }
    }

    /**
     * set where clause (filter)
     *
     * The syntax is as follows:
     * column1=value1,column2=value2,...,columnN=valueN
     *
     * The alternative is as follows:
     * array(array(0=>column1,1=>value1,2=>operator1),...)
     * Where "operator" can be one of the following:
     * '=', 'LIKE', '<', '>', '!=', '<=', '>='
     *
     * To unset the where clause, call this function without
     * providing a parameter.
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  public
     * @param   string|array  $where
     * @return  bool
     */
    function setWhere($where = null)
    {
        if (empty($where)) {

            $this->where = array();
            return true;

        } else if (is_string($where)) {

            /* get primary key */
            if (is_object($this->structure) && !empty($this->table)) {
                $primaryKey = $this->structure->getPrimaryKey($this->table);
            } else {
                $primaryKey = '';
            }

            if (empty($primaryKey)) {
                trigger_error("Unable to get primary key for current table while setting 'where'. Result might be corrupt.", E_USER_NOTICE);
            }

            /**
             * Note:
             * when searching for a primary key, page numbering is irrelevant
             * so it needs to be reseted
             */
            if (strpos(strtolower($where), strtolower($primaryKey).'=') > -1) {
                $this->page = 0;
            }

            $this->where = $where;
            return true;

        } else if (is_array($where)) {

            /* get primary key */
            if (is_object($this->structure) && !empty($this->table)) {
                $primaryKey = $this->structure->getPrimaryKey($this->table);
            } else {
                $primaryKey = '';
            }

            if (empty($primaryKey)) {
                trigger_error("Unable to get primary key for current table while setting 'where'. Result might be corrupt.", E_USER_NOTICE);
            }

            /**
             * Note:
             * This array scan is not much of a performance issue, as $where
             * has in up to 90% of all cases at most 1 item.
             */
            assert('!isset($temp); // Cannot redeclare variable $temp');
            foreach ($where as $temp)
            {
                if (isset($temp[0]) && strcasecmp($temp[0], $primaryKey) === 0) {
                    $this->page = 0;
                    break;
                }
            } /* end foreach */
            unset($temp);

            $this->where = $where;
            return true;

        } else {

            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($where)), E_USER_WARNING);
            return false;

        }
    }

    /**
     * get the currently set where clause
     *
     * Returns the current where clause, or bool(false)
     * if none has been set yet.
     *
     * @access  public
     * @return  bool(false)|string|array
     */
    function getWhere()
    {
        if (empty($this->where)) {
            return false;
        } else if (is_string($this->where) || is_array($this->where)) {
            return $this->where;
        } else {
            /* error: no where clause has been selected, yet */
            return false;
        }
    }

    /**
     * resolve foreign keys
     *
     * Returns bool(true) on success and bool(false) on error.
     *
     * @access  private
     * @return  bool
     */
    function _getForeignKeys()
    {
        /* get a list of all foreign keys */
        if (is_object($this->structure)) {
            $foreign_keys = $this->structure->getForeignKeys($this->table);
            if (empty($foreign_keys)) {
                /* no foreign keys found */
                return true;
            }
        } else {
            /* error: structure not initialized */
            return false;
        }

        /* loop through foreign keys */
        foreach ($foreign_keys as $key => $foreign_table)
        {
            /* if is foreign key of type select ... */
            if (strcasecmp($this->structure->getType($this->table, $key), 'select') == 0) {

                /* get default values */
                $default =& $this->content[strtoupper($key)]['DEFAULT'];

                /* 1) if default is not set */
                if (is_null($default)) {

                    assert('!isset($defaultList);');
                    $defaultList = $this->db->get("${foreign_table}.*.${key}");
                    $default     = array();
                    assert('!isset($defaultValue);');
                    foreach ($defaultList as $defaultValue)
                    {
                        assert('is_scalar($defaultValue); // unexpected value: $defaultValue');
                        $default[$defaultValue] = $defaultValue;
                    }
                    unset($defaultList, $defaultValue);

                /* 2) or default is an array of table references */
                } else if (is_array($default) && count($default) === 1) {

                    assert('!isset($col1) && !isset($col2) && !isset($defaultKeys) && !isset($defaultValues);');
                    foreach ($default as $col1 => $col2)
                    {
                        if ($this->structure->isColumn($foreign_table, $col1)) {
                            $defaultKeys = $this->db->get("${foreign_table}.*.${col1}");
                        } else {
                            $defaultKeys = array();
                            trigger_error("The foreign key '${key}' in table '${table}' references a non-existing column '${col1}' in table '${foreign_table}'.",E_USER_ERROR);
                        }
                        if ($this->structure->isColumn($foreign_table, $col2)) {
                            $defaultValues = $this->db->get("${foreign_table}.*.${col2}");
                        } else {
                            $defaultValues = array();
                            trigger_error("The foreign key '${key}' in table '${table}' references a non-existing column '${col2}' in table '${foreign_table}'.",E_USER_ERROR);
                        }
                        break;

                    } /* end foreach */
                    /* check unique constraint */
                    if (count($defaultKeys) !== count(array_unique($defaultKeys))) {
                        trigger_error("The column '${col1}' in table '${foreign_table}' referenced as foreign key from table '${table}' contains ambigious values. Possible reasons are that your data is inconsistent or your database scheme contains errors.", E_USER_WARNING);
                    }
                    unset($col1, $col2);
                    assert('is_array($defaultKeys) && is_array($defaultValues) && count($defaultKeys) === count($defaultValues);');
                    assert('!isset($i); /* Unable to redeclare $i */');
                    $default = array();
                    assert('empty($content[strtoupper($key)]["DEFAULT"]); /* Default settings are not empty */');
                    $default = array_combine($defaultKeys, $defaultValues);
                    unset($defaultKeys, $defaultValues);

                /* 3) ignore empty values */
                } else if (empty($default)) {

                    continue;

                /* 4) error: encountered an unexpected value */
                } else {

                    return false;

                }
                unset($foreign_table, $foreign_keys);
            } /* end if */
        } /* end foreach */
        unset($key);
        /* success */
        return true;
    }

    /**
     * get rows
     *
     * Returns an empty array on error.
     *
     * @access  private
     * @return  array
     */
    function _getRows()
    {

        if (!is_string($this->table)) {
            trigger_error("Unable to retrieve resultset. No table has been selected.", E_USER_WARNING);
            return array();
        } else if (!is_object($this->db)) {
            trigger_error("Unable to retrieve resultset. No database connection available.", E_USER_WARNING);
            return array();
        }

        /* get last page */
        if (empty($this->where)) {
            $this->lastPage = $this->db->length($this->table);
        } else {
            $this->lastPage = $this->db->length($this->table, $this->where);
        }
        if (!is_int($this->lastPage)) {
            /* error: e.g. table does not exist */
            $this->lastPage = 0;
            return array();
        }

        /* make sure the user can't go further than the last page */
        if ($this->page > $this->lastPage) {
            if ($this->lastPage > 0) {
                $this->page = (int) $this->lastPage - 1;
            } else {
                $this->page = 0;
            }
        }

        /* get resultset */
        $rows = $this->db->get($this->table, $this->where, $this->orderBy, $this->page, $this->entries, $this->desc);

        if (empty($rows)) {
            $this->currentIndex = 0;
            return array();
        } else {
            $this->currentIndex = array_shift(array_keys($rows));
            /* resolve page number */
            if (!empty($this->where)) {
                assert('!isset($temp);');
                $temp = $this->db->get($this->table.'.*.'.$this->structure->getPrimaryKey($this->table), $this->where, $this->orderBy, 0, 0, $this->desc);
                assert('is_array($temp);');
                $temp = array_search($this->currentIndex, $temp);
                assert('is_int($temp);');
                if ($temp === false) {
                    $temp = 0;
                }
                $_SESSION['page'][$this->table] = $temp;
                $this->page = $temp;
                unset($temp);
            }
        }

        /* get offset */
        $length = count($rows);
        if ($length > 0) {
            $offsetPage = $this->page + $length;
            $this->template->insert('OFFSET_PAGE', $offsetPage);
            if ($this->firstPage > $offsetPage) {
                $this->firstPage = $offsetPage;
            }
        } else {
            $this->template->insert('OFFSET_PAGE', 0);
        }

        /* prepare files and images
         *
         * @todo  review this section and find a better way to do this (scheduled for: v3.0)
         */
        $columnList = $this->structure->getFiles($this->table);
        if (!empty($columnList)) {
            global $YANA;
            assert('!isset($dbBlob); // Cannot redeclare var $dbBlob');
            $dbBlob = $YANA->getVar('DBBLOB');
            foreach ($rows as $a => $b)
            {
                foreach ($columnList as $column)
                {
                    $column = strtoupper($column);
                    if (!empty($b[$column])) {
                        $string = new String($b[$column]);
                        $string->replace($dbBlob, '');
                        /* applies to images only: use thumbnails instead of original image */
                        $string->replace('image.', 'thumb.');
                        /* encrypt path */
                        $string->encrypt('xor', YANA_INSTANCE_ID . session_id());
                        /* unbox string */
                        $string = $string->get();

                        /* add file to SESSION-vars */
                        if (!isset($_SESSION['filelist']) || !is_array($_SESSION['filelist'])) {
                            $_SESSION['filelist'] = array();
                            $i = false;
                        } else {
                            $i = array_search($string, $_SESSION['filelist']);
                        }
                        if ($i === false) {
                            $i = count($_SESSION['filelist']) + 1;
                            $_SESSION['filelist'][$i] = $string;
                        }
                        $rows[$a][$column] = $i;
                    }
                } /* end foreach */
            } /* end foreach */
        } /* end if */

        return $rows;
    }

    /**
     * get data from posted form
     *
     * This function can be used to collect the data submitted
     * by a form, which was created using the Yana form generator.
     *
     * It returns an array containing the converted form data or
     * bool(false) on error.
     *
     * If the argument $data is not provided, the function automatically
     * takes it from the $_POST array instead.
     *
     * Example of usage:
     * <code>
     * $args = GuiCreator::getFormdata(GUI_NEW, "guestbook", "guestbook");
     * </code>
     *
     * @access  public
     * @static
     * @param   int                   $form    choose the form, the data came from (GUI_NEW or GUI_EDIT)
     * @param   string|StructureFile  $source  name of the database structure file to use
     * @param   string                $table   name of the database table, the data is related to
     * @param   array                 $data    provide the form data
     * @return  array|bool(false)
     */
    function getFormdata($form, $source, $table, $data = null)
    {
        /**
         * Input checking
         */
        assert('!isset($_chk);');
        assert('!isset($_types);');
        $_types = array(
        /* argument 1 */ IS_INT,
        /* argument 2 */ IS_STRING|IS_OBJECT,
        /* argument 3 */ IS_STRING,
        /* argument 4 */ IS_ARRAY|IS_NULL
        );
        $_chk = checkArgumentList(func_get_args(), $_types, __CLASS__.'::'.__FUNCTION__);
        if ($_chk !== true) {
            trigger_error($_chk, E_USER_WARNING);
            return false;
        } else {
            unset($_chk, $_types);
        }

        if ($form !== GUI_NEW && $form !== GUI_EDIT) {
            trigger_error("The selected form is not supported.\n\t\tPlease choose one of the following constants: GUI_NEW, GUI_EDIT.", E_USER_WARNING);
            return false;
        }

        if (is_null($data)) {
            $data = $_POST;
        }
        $data = Hashtable::changeCase($data, CASE_LOWER);

        /* load structure file */
        if (is_string($source)) {

            $structure = new StructureFile($source);

            /* check if file exists */
            if (!$structure->exists()) {
                trigger_error("Invalid structure file '${source}'.", E_USER_WARNING);
                return false;
            } else {
                $structure->read();
            }

        } else if (!is_a($source, 'StructureFile')) {

            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 2, 'Instance of StructureFile', get_class($source)), E_USER_WARNING);
            return false;

        } else {

            $structure =& $source;
            $structure->read();

        }

        /* check if table exists */
        if (!$structure->isTable($table)) {
            trigger_error("Invalid table name '${table}'.", E_USER_WARNING);
            return false;
        }

        /* lowercase file array */
        if (isset($_FILES) && is_array($_FILES)) {
            $FILES = Hashtable::changeCase($_FILES, CASE_LOWER);
        } else {
            $FILES = array();
        }

        /* declare $result */
        assert('!isset($result); // Cannot redeclare $result');
        $result = array(); /* used to store returned vallue */

        /* declare $selectedEntries */
        assert('!isset($selectedEntries); // Cannot redeclare $selectedEntries');
        $selectedEntries = array();

        /* declare $prefixes */
        assert('!isset($prefixes); // Cannot redeclare $prefixes');
        $prefixes = array();

        /* get list of selected entries */
        if ($form === GUI_EDIT) {

            /* check if any entries have been selected */
            if (!isset($data['selected_entries'])) {
                return array();
            } else {
                $selectedEntries = $data['selected_entries'];
                settype($selectedEntries, 'array');
            }

            /* fill $result and $prefixes */
            assert('!isset($entry); // Cannot redeclare $entry');
            foreach ($selectedEntries as $entry)
            {
                if (is_scalar($entry)) {
                    $entry = strtolower($entry);
                    $result[$entry] = array();
                    $prefixes[$entry] = "id_${entry}_";
                }
            }
            unset($entry);

        } else {

            $prefixes = array("");

        } /* end if */

        /* iterate through columns */
        assert('!isset($column); // Cannot redeclare $column');
        foreach ($structure->getColumns($table) as $column)
        {
            $column = strtolower($column);
            $type = strtolower($structure->getType($table, $column));

            switch ($form)
            {
                case GUI_NEW:

                    /* check if column can be edited */
                    if (!$structure->isVisible($table, $column, 'new')) {
                        continue 2;
                    } /* end if */

                    $current = array(&$result);
                    $i = $column;

                break;

                case GUI_EDIT:

                    /* check if column can be edited */
                    if (!$structure->isEditable($table, $column, 'edit')) {
                        continue 2;
                    } /* end if */

                    $current =& $result;
                    $i = "";

                break;

                default:
                    return false;
                break;
            } /* end switch */

            /* create result rows
             *
             * Note: GUI_NEW has only one row as
             * only one row can be added at a time
             */
            assert('!isset($id); // Cannot redeclare $id');
            foreach (array_keys($current) as $id)
            {
                /* initialize where necessary */
                if (!isset($current[$id]) || !is_array($current[$id])) {
                    $current[$id] = array();
                }

                /* get form name */
                $i = $prefixes[$id] . $column;

                /* initialize where necessary */
                switch ($type)
                {
                    case 'boolean': case 'bool':
                        if (isset($data[$i]) && $data[$i]) {
                            $current[$id][$column] = true;
                        } else {
                            $current[$id][$column] = false;
                        }
                    break;

                    case 'time':
                        if (isset($data[$i.'_day'])) {
                            $current[$id][$column] = DisplayUtility::toTimestamp($i.'_', $data);
                        }
                    break;

                    case 'image': case 'file':
                        if (!empty($FILES[$i]['name'])) {
                            $current[$id][$column] = $i;
                        }
                    break;

                    case 'array':
                        /* get names and values */
                        assert('!isset($names); // Cannot redeclare $names');
                        if (isset($data[$i.'_names']) && is_array($data[$i.'_names'])) {
                            $names = $data[$i.'_names'];
                        } else {
                            $names = array();
                        }
                        assert('!isset($values); // Cannot redeclare $values');
                        if (isset($data[$i.'_values']) && is_array($data[$i.'_values'])) {
                            $values = $data[$i.'_values'];
                        } else {
                            $values = array();
                        }
                        if (empty($names)) {
                            unset($names, $values);
                            continue;
                        }

                        /* build array */
                        assert('!isset($param); // Cannot redeclare $param');
                        $param = array();
                        if (count($names) === count($values)) {
                            $param = array();
                            assert('!isset($j); // Cannot redeclare $j');
                            for ($j = 0; $j < count($names); $j++)
                            {
                                if (!empty($names[$j])) {
                                    $values[$j] = untaintInput($values[$j], "string", 255, YANA_ESCAPE_LINEBREAK);
                                    Hashtable::set($param, $names[$j], $values[$j]);
                                }
                            }
                            unset($j);
                        }
                        unset($names, $values);

                        if (empty($param)) {
                            unset($param);
                            continue;
                        }

                        $current[$id][$column] = $param;
                        unset($param);
                    break;

                    case 'text':
                        if (isset($data[$i])) {
                            $current[$id][$column] = untaintInput($data[$i], "string", 0, YANA_ESCAPE_USERTEXT);
                        }
                    break;

                    default:
                        if (isset($data[$i])) {
                            $current[$id][$column] = untaintInput($data[$i], "string", 0, YANA_ESCAPE_LINEBREAK);
                        }
                    break;
                } /* end switch */
            } /* end foreach */
            unset($id);

        } /* end foreach */
        unset($column);

        return $result;
    }

    /**
     * smarty function create
     *
     * This is a generator function to create dynamic HTML forms from database
     * schema files and database values.
     * Forms with several layouts can be generated to view, edit, insert, delete
     * or search database values.
     *
     * If necessary it communicates with the database to retrieve values.
     * This communication is limited to Select-Statements. No data is changed.
     *
     * For security reasons this function does NOT provide functionality to
     * write changes made in the forms to the database on it's own.
     * This might have introduced the possibility to compromise the security settings
     * of third-party plugIns and thus has never been implemented and never will be.
     *
     * Instead you need to provide the name of a function in your plugin,
     * that will handle the requests. This gives you the chance to do additional
     * security checks and filter the provided data as you see fit.
     *
     * <pre>
     * This function takes the following arguments:
     *
     * string  $template        (mandatory) template that should be used (view_details, view_seperated, view, new, edit, search)
     * string  $file            (mandatory) path and name of structure file
     * string  $table           (mandatory) table to choose from structure file
     * string  $where           (optional)  sequence for SQL-where clause <FIELDNAME1>=<VALUE1>[,<FIELDNAME2>=<VALUE2>[,...]]
     * string  $sort            (optional)  name of column to sort entries by
     * boolean $desc            (optional)  sort entries descendant (true) or ascendant (false)
     * integer $page            (optional)  number of 1st entry to show
     * integer $entries         (optional)  number of entries to show on each page
     * boolean $titles          (optional)  should fieldnames be shown (true=yes, false=no)
     * string  $on_new          (optional)  name of action (plugin-function) to activate in order to compute the event
     * string  $on_edit         (optional)  name of action (plugin-function) to activate in order to compute the event
     * string  $on_delete       (optional)  name of action (plugin-function) to activate in order to compute the event
     * string  $on_search       (optional)  name of action (plugin-function) to activate in order to compute the event
     * boolean $advanced_search (optional)  should advanced search form be shown (true=yes, false=no)
     * </pre>
     *
     * Example of usage:
     * <code>
     * [%create template="view" file="guestbook" table="guestbook" sort="guestbook_date" desc="true"%]
     * </code>
     *
     * @access  public
     * @static
     * @param   array   $params
     * @return  string|bool(false)
     */
    function create($params)
    {
        /* type checking */
        if (!is_array($params)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'Array', gettype($params)), E_USER_WARNING);
            return false;
        }

        $isSearch = false; /* true when using search template */

        /**
         * initialize session
         */
        if (!isset($_SESSION['sort'])    || !is_array($_SESSION['sort'])) {
            $_SESSION['sort'] = array();
        }
        if (!isset($_SESSION['desc'])    || !is_array($_SESSION['desc'])) {
            $_SESSION['desc'] = array();
        }
        if (!isset($_SESSION['page'])    || !is_array($_SESSION['page'])) {
            $_SESSION['page'] = array();
        }
        if (!isset($_SESSION['where'])   || !is_array($_SESSION['where'])) {
            $_SESSION['where'] = array();
        }
        if (!isset($_SESSION['entries']) || !is_array($_SESSION['entries'])) {
            $_SESSION['entries'] = array();
        }

        /**
         * create new GuiCreator-instance
         */
        $GENERATOR = new GuiCreator();
        $test      = true;

        /**
         * Parameter 'template'
         */
        if (!isset($params['template'])) {
            /* this parameter is mandatory */
            return false;
        }
        $template = (string) $params['template'];
        $template = strtolower($template);
        if (isset($GENERATOR->tmplMap[$template])) {
            $test = $GENERATOR->setTemplate($template);
            if ($test === true) {
                $GENERATOR->template->insert('NAMESPACE', $template);
            } else {
                /* error: template was invalid */
                return false;
            }
        } else {
            /* error: unknown template*/
            return false;
        }
        if ($template === 'search' || $template === 'adv_search') {
            $isSearch = true;
        }
        /**
         * Note: $template will be used later ...
         * it is intentionally not unset here.
         */

        /**
         * Parameter checking:
         * 'table', 'file', 'on_edit', 'on_delete', 'on_new', 'on_search'
         */
        if (isset($params['table'])) {
            $test = $test && $GENERATOR->setTable(        $params['table']     );
        } else {
            /* this parameter is mandatory */
            return false;
        }
        if (isset($params['file'])) {
            $test = $test && $GENERATOR->setFile(         $params['file']      );
        } else {
            /* this parameter is mandatory */
            return false;
        }
        if (isset($params['on_edit'])) {
            $test = $test && $GENERATOR->setEditAction(   $params['on_edit']   );
        }
        if (isset($params['on_delete'])) {
            $test = $test && $GENERATOR->setDeleteAction( $params['on_delete'] );
        }
        if (isset($params['on_new'])) {
            $test = $test && $GENERATOR->setNewAction(    $params['on_new']    );
        }
        if (isset($params['on_search'])) {
            $test = $test && $GENERATOR->setSearchAction( $params['on_search'] );
        }

        /**
         * Parameter 'titles'
         */
        if (isset($params['titles']) && ($params['titles'] == 'hidden' || $params['titles'] == 'false' || (bool) $params['titles'] == false)) {
            $test = $test && $GENERATOR->enableTitles(false);
        } else {
            $test = $test && $GENERATOR->enableTitles(true);
        }

        /**
         * Parameter 'advanced_search'
         */
        if (!isset($_SESSION['advanced_search']) || !is_array($_SESSION['advanced_search'])) {
            $_SESSION['advanced_search'] = array();
        }
        if (isset($_GET['advanced_search'])) {
            if ($_GET['advanced_search']) {
                $advanced_search = true;
            } else {
                $advanced_search = false;
            }
        } else if (isset($_SESSION['advanced_search'][$GENERATOR->getTable()])) {
            if ($_SESSION['advanced_search'][$GENERATOR->getTable()]) {
                $advanced_search = true;
            } else {
                $advanced_search = false;
            }
        } else if (isset($params['advanced_search'])) {
            $advanced_search = $params['advanced_search'];
        } else {
            $advanced_search = false;
        }
        if ($advanced_search) {
            $_SESSION['advanced_search'][$GENERATOR->getTable()] = true;
            $test = $test && $GENERATOR->enableAdvancedSearch(true);
        } else {
            $_SESSION['advanced_search'][$GENERATOR->getTable()] = false;
            $test = $test && $GENERATOR->enableAdvancedSearch(false);
        }
        unset($advanced_search);

        /**
         * Parameter checking:
         * 'sort', 'desc'
         */

        /* get parameters from URL or session */
        if (isset($_GET['sort'])) {
            $sort = $_GET['sort'];
        } else if (isset($_SESSION['sort'][$GENERATOR->getTable()])) {
            $sort = $_SESSION['sort'][$GENERATOR->getTable()];
        } else if (isset($params['sort'])) {
            $sort = $params['sort'];
        } else {
            $sort = "";
        }

        /* sort defaults to the primary key */
        if (empty($sort)) {
            $sort = $GENERATOR->structure->getPrimaryKey($GENERATOR->getTable());
        }
        if (isset($_GET['desc'])) {
            if ($_GET['desc']) {
                $desc = true;
            } else {
                $desc = false;
            }
        } else if (isset($_SESSION['desc'][$GENERATOR->getTable()])) {
            $desc = $_SESSION['desc'][$GENERATOR->getTable()];
        } else if (isset($params['desc'])) {
            $desc = $params['desc'];
        } else {
            $desc = false;
        }

        /* type casting */
        /* settype to STRING
         *            BOOLEAN
         */
        $sort = (string) $sort;
        $desc = (bool)   $desc;

        /* apply settings */
        if (!$GENERATOR->setSort($sort, $desc)) {
            return false;
        } else {
            $_SESSION['sort'][$GENERATOR->getTable()] = $sort;
            $_SESSION['desc'][$GENERATOR->getTable()] = $desc;
        }
        unset($sort, $desc);

        /**
         * Parameter checking:
         * 'page', 'entries'
         */

        /* get parameters from URL or session */
        if (isset($_GET['page'])) {
            $page = $_GET['page'];
        } else if (isset($_SESSION['page'][$GENERATOR->getTable()])) {
            $page = $_SESSION['page'][$GENERATOR->getTable()];
        } else if (isset($params['page'])) {
            $page = $params['page'];
        } else {
            $page = 0;
        }
        if (!empty($_GET['entries'])) {
            $entries = $_GET['entries'];
        } else if (!empty($_SESSION['entries'][$GENERATOR->getTable()])) {
            $entries = $_SESSION['entries'][$GENERATOR->getTable()];
        } else if (!empty($params['entries'])) {
            $entries = $params['entries'];
        } else {
            $entries = 5;
        }

        /* settype to INTEGER
         *            INTEGER
         */
        $page    = (int) $page;
        $entries = (int) $entries;

        $_SESSION['page'][$GENERATOR->getTable()]    = $page;
        $_SESSION['entries'][$GENERATOR->getTable()] = $entries;
        $test = $test && $GENERATOR->setLimits($page, $entries);
        unset($page, $entries);

        /**
         * Parameter 'where'
         */
        if ($isSearch === true) {

            $where = array();

            /**
             * The next passage applies to columns of type TIME
             * only. To search for a date / time a range needs to be
             * created.
             *
             * The following code will create such a range when necessary.
             */
            assert('!isset($fieldname); /* cannot redeclare variable $fieldname */');
            foreach ($GENERATOR->structure->getColumns($GENERATOR->getTable()) as $column)
            {
                $fieldname = strtoupper($column);
                /**
                 * If:
                 * 1) the filed is activated AND
                 * 2) type == time
                 * Then:
                 * 3) create range
                 *
                 * Note: the "simple" search form has only columns of type text.
                 */
                if (!$GENERATOR->getAdvancedSearch()) {
                    if (!empty($_POST[strtoupper($fieldname)])) {
                        $where[] = array($fieldname, '%'.$_POST[strtoupper($fieldname)].'%', 'like');
                    }                    
                } else if ($_POST[strtolower('is_active_'.$fieldname)] === 'true') {

                    if (strcasecmp($GENERATOR->structure->getType($GENERATOR->getTable(), $fieldname), 'time') === 0) {

                        assert('!isset($time) && !isset($time_stamp);');

                        global $YANA;

                        /* start time*/
                        $time = array();
                        $time['Y'] = (int) $YANA->getRequestVar("${fieldname}_START_Year");
                        $time['M'] = (int) $YANA->getRequestVar("${fieldname}_START_Month");
                        $time['D'] = (int) $YANA->getRequestVar("${fieldname}_START_Day");
                        $time['h'] = (int) $YANA->getRequestVar("${fieldname}_START_Hour");
                        $time['m'] = (int) $YANA->getRequestVar("${fieldname}_START_Minute");
                        $time['s'] = (int) $YANA->getRequestVar("${fieldname}_START_Second");

                        /* create filter rule for start time */
                        if (!empty($time)) {
                            $time_stamp = mktime($time['h'], $time['m'], $time['s'], $time['M'], $time['D'], $time['Y']);
                            if (!empty($time_stamp) && is_int($time_stamp)) {
                                assert('is_array($where); /* unexpected input $where */');
                                $where[] = array($fieldname, $time_stamp, '>=');
                            }
                        }

                        /* end time*/
                        $time = array();
                        $time['Y'] = (int) $YANA->getRequestVar("${fieldname}_END_Year");
                        $time['M'] = (int) $YANA->getRequestVar("${fieldname}_END_Month");
                        $time['D'] = (int) $YANA->getRequestVar("${fieldname}_END_Day");
                        $time['h'] = (int) $YANA->getRequestVar("${fieldname}_END_Hour");
                        $time['m'] = (int) $YANA->getRequestVar("${fieldname}_END_Minute");
                        $time['s'] = (int) $YANA->getRequestVar("${fieldname}_END_Second");

                        /* create filter rule for start time */
                        if (!empty($time)) {

                            /**
                             * Note:
                             * to create a valid range, the hours, minutes and
                             * seconds need to be set to the end of the day.
                             *
                             * This does not apply to the start time, because
                             * the default values are int(0), which happens to
                             * be interpreted as midnight (start of the day).
                             * So we will leave these to the defaults.
                             */
                            if (empty($time['h'])) {
                                $time['h'] = 23;
                            }
                            if (empty($time['m'])) {
                                $time['m'] = 59;
                            }
                            if (empty($time['s'])) {
                                $time['s'] = 59;
                            }

                            $time_stamp = mktime($time['h'], $time['m'], $time['s'], $time['M'], $time['D'], $time['Y']);
                            if (!empty($time_stamp) && is_int($time_stamp)) {
                                assert('is_array($where); /* unexpected input $where */');
                                $where[] = array($fieldname, $time_stamp, '<=');
                            }
                        }

                        /* success */
                        unset($time, $time_stamp);

                    /* for other create match */
                    } else {
                        if (!empty($_POST[strtoupper($fieldname)])) {
                            $where[] = array($fieldname, '%'.$_POST[strtoupper($fieldname)].'%', 'like');
                        }
                    }

                } else {

                    continue;

                } /* end if */
            } /* end foreach */
            unset($fieldname, $column);

            /* set where clause */
            if (empty($where) && !empty($_SESSION['where'][$GENERATOR->getTable()])) {

                $where = $_SESSION['where'][$GENERATOR->getTable()];
                $test = $test && $GENERATOR->setWhere($where);

            } else if (!empty($where)) {

                $_SESSION['where'][$GENERATOR->getTable()] = $where;
                $test = $test && $GENERATOR->setWhere($where);

            } else {
                /* where clause is empty - nothing to do */
            }

            unset($where);

        } else if (isset($params['where'])) {

            $test = $test && $GENERATOR->setWhere($params['where']);

        }

        if ($test !== true) {
            /* error: input was invalid */
            return false;
        }

        /**
         * Begin business logic
         */
        $GENERATOR->template->insert('PRIMARY_KEY', $GENERATOR->structure->getPrimaryKey($GENERATOR->getTable()));
        $GENERATOR->_getForeignKeys();
        switch ($template)
        {
            case 'adv_search':
                $GENERATOR->enableAdvancedSearch();
            /* fall through */

            case 'search':
                /* notify the template, that we are in a search block */
                $GENERATOR->template->insert('IS_SEARCH', true);

                /* switch to advanced search template when necessary */
                if ($GENERATOR->getAdvancedSearch()) {
                    $GENERATOR->setTemplate(GUI_ADV_SEARCH);
                } else {
                    assert('$GENERATOR->getTemplate() === $GENERATOR->fileMap[GUI_SEARCH]; /* unexpected result: search template not selected */');
                    /* keep current template (we assume it is GUI_SEARCH) */
                }
            /* fall through */

            case 'view_details':
            case 'view_seperated':
            case 'view':
            case 'edit':
                $GENERATOR->template->insert('ROWS', $GENERATOR->_getRows());
                $GENERATOR->template->insert('LIST_OF_ENTRIES', $GENERATOR->_getListOfEntries());
            /* fall through */

            case 'new':
                /* intentionally left blank */
            break;

            default:
                trigger_error("Template-Syntaxerror, unknown template for GUI-Generator: '${template}'.", E_USER_WARNING);
                return "";
            break;
        } /* end switch */

        return $GENERATOR->template->get(null, false);

    }

    /**
     * create links to other pages
     *
     * @access  private
     * @return  array
     */
    function _getListOfEntries()
    {
        if ($this->lastPage > ($this->entries * 2)) {
            $list_of_pages = array();
            $dots          = false;

            $is_too_long = $this->lastPage > (10 * $this->entries);
            for ($k = 0; $k < ceil( $this->lastPage / $this->entries ); $k++)
            {
                /**
                 * if more than 10 pages exist and current page is not first page or last page
                 * and is not current page or previous or next 3 pages
                 */
                $is_near_current = (floor($this->page / $this->entries) - 3) < $k && $k < (floor($this->page / $this->entries) + 3);

                $is_first_or_last = $k <= 1 || $k >= (ceil($this->lastPage / $this->entries) - 2 );

                if ($is_too_long && !$is_first_or_last && !$is_near_current) {
                    /* this marks an elipsis */
                    if ($dots === false) {
                        $dots = true;
                        array_push($list_of_pages, false);
                    } else {
                        /* ignore this page */
                        continue;
                    }
                } else {
                    $first = ($k * $this->entries);
                    if (($k + 1) * $this->entries < $this->lastPage) {
                        $last = ($k + 1) * $this->entries;
                    } else {
                        $last = $this->lastPage;
                    }
                    assert('is_array($list_of_pages); /* unexpected input $list_of_pages */');
                    $list_of_pages[] = array('FIRST' => ($first + 1), 'LAST' => $last);
                    if ($is_near_current) {
                        $dots = false;
                    }
                }
            } /* end for */

            assert('is_array($list_of_pages); /* Unexpected output */');
            return $list_of_pages;
        } else {
            /* only one page exists, so the result is empty */
            return array();
        }
    }

    /**
     * self test
     *
     * This function exists only for debugging purposes.
     * Do not uncomment in productive environments.
     *
     * @access  public
     * @static
     * @param   int  $nr  number of test to take
     * @return  bool
     * @ignore
     * /
    function test($nr)
    {
        switch ($nr)
        {
            case 1:
                $gui = new GuiCreator();
                // setting / getting file
                $gui->setFile('GUESTBOOK');
                $test = $gui->getFile();
                if ($test !== "guestbook") {
                    assert('false; // "set/get file" test failed');
                    return false;
                }
                $gui->setTable('GUESTBOOK');
                // setting / getting table
                $test = $gui->getTable();
                if ($test !== "guestbook") {
                    assert('false; // "set/get table" test failed');
                    return false;
                }
                // setting / getting template
                $gui->setTemplate('edit');
                $test1 = $gui->getTemplate();
                $gui->setTemplate($test1);
                $test2 = $gui->getTemplate();
                $gui->setTemplate(GUI_EDIT);
                $test3 = $gui->getTemplate();
                if (!is_string($test1) || $test1 !== $test2 || $test2 !== $test3) {
                    assert('false; // "set/get template" test failed');
                    return false;
                }
                // setting / getting action
                $gui->setAction('EDIT', 'SITEMAP');
                $test = $gui->getAction('EDIT');
                if ($test !== 'sitemap') {
                    assert('false; // "set/get action" test failed');
                    return false;
                }
                // setting / getting bool values
                $gui->enableTitles();
                $gui->enableAdvancedSearch();
                $test1 = $gui->getTitles();
                $test2 = $gui->getAdvancedSearch();
                if ($test1 !== true || $test2 !== true) {
                    assert('false; // "set/get bool values" test failed');
                    return false;
                }
                // setting / getting sorting
                $test1 = $gui->setSort('foo');
                $gui->setSort('GUESTBOOK_name', true);
                $test2 = $gui->getSort();
                $test3 = $gui->isDescending();
                if ($test1 !== false || $test2 !== 'guestbook_name' || $test3 !== true) {
                    assert('false; // "set/get sorting" test failed');
                    return false;
                }
                // setting / getting page + entries
                // this should output 2 E_USER_NOTICES
                $gui->setLimits(-1, 0);
                $test1 = $gui->getPage();
                $gui->setLimits(1, 2);
                $test2 = $gui->getPage();
                $test3 = $gui->getEntriesPerPage();
                if ($test1 !== 0 || $test2 !== 1 || $test3 !== 2) {
                    assert('false; // "set/get page+entries" test failed');
                    return false;
                }
                // setting / getting where clause
                $gui->setWhere("a=b");
                $test1 = $gui->getWhere();
                $gui->setWhere(array());
                $test2 = $gui->getWhere();
                if ($test1 !== "a=b" || $test2 !== false) {
                    assert('false; // "set/get where" test failed');
                    return false;
                }
                $gui->setFile('user');
                $gui->setTable('user');
                $test = $gui->_getForeignKeys();
                if ($test !== true) {
                    assert('false; // "get foreign keys" test failed');
                    return false;
                }
                $test = GuiCreator::create(array('file'=>'guestbook','template'=>'edit','table'=>'guestbook'));
                if (!is_string($test) || empty($test)) {
                    assert('false; // test failed');
                    return false;
                }
                return true;
            break;
        }
    } /* end of function */

}

?>
