<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * @ignore
 */

/**
 * Singleton Skin
 *
 * @access      public
 * @package     yana
 * @subpackage  core
 *
 * @ignore
 */
class Skin extends Singleton
{

    /**#@+
     * @ignore
     * @access  private
     */

    /** @var string    */ var $directory  = "";
    /** @var dirStream */ var $dirStream  = null;
    /** @var SML[]     */ var $files      = array();
    /** @var array     */ var $value      = array();

    /**#@-*/

    /**
     * Singleton Constructor
     *
     * @param  string     $directory
     * @param  dirStream  $dirStream
     * @param  dirStream  $skinDir
     * @param  dirStream  $defaultDir
     *
     * @ignore
     */
    function Skin($directory, $dirStream, $skinDir, $defaultDir)
    {
        $this->checkSingletonConstraint();
        assert('is_string($directory) && is_dir($directory);');
        /* settype to STRING */
        $directory = (string) $directory;

        $this->directory = $directory;
        $this->dirStream = $dirStream;
        $this->dirStream->read();

        /* Load Defaults first */
        if ($defaultDir->getPath() != $skinDir->getPath()){
            $content = $defaultDir->get();
            assert('!isset($filename); /* cannot redeclare variable $filename */');
            foreach ($content as $filename)
            {
                $this->files[$filename] = new SML($defaultDir->getPath().$filename, CASE_UPPER);
                $this->files[$filename]->read();
                $REF = $this->files[$filename]->get("*");
                assert('!isset($key); /* cannot redeclare variable $key */');
                assert('!isset($element); /* cannot redeclare variable $element */');
                foreach ($REF as $key => $element)
                {
                    if (!empty($element['FILE'])) {
                        $this->value[$key]['FILE'] = "../../".$defaultDir->getPath().$element['FILE'];
                    }
                    if (!empty($element['STYLE'])) {
                        skin::_fallback($this->value[$key]['STYLE'], $element['STYLE'], $defaultDir->getPath());
                    }
                    if (!empty($element['SCRIPT'])) {
                        skin::_fallback($this->value[$key]['SCRIPT'], $element['SCRIPT'], $defaultDir->getPath());
                    }
                    if (!empty($element['LANGUAGE'])) {
                        skin::_fallback($this->value[$key]['LANGUAGE'], $element['LANGUAGE'], false);
                    }
                } /* end foreach */
                unset($key, $element);
            } /* end foreach */
            unset($filename);
        }

        /* Overwrite Defaults where needed */
        $content = $skinDir->get();
        assert('!isset($filename); /* cannot redeclare variable $filename */');
        foreach ($content as $filename)
        {
            $this->files[$filename] = new SML($skinDir->getPath().$filename, CASE_UPPER);
            $this->files[$filename]->read();
            $REF = $this->files[$filename]->get("*");
            assert('!isset($key); /* cannot redeclare variable $key */');
            assert('!isset($element); /* cannot redeclare variable $element */');
            foreach ($REF as $key => $element)
            {
                if (!empty($element['FILE'])) {
                    $this->value[$key]['FILE'] = $element['FILE'];
                }
                if (!empty($element['STYLE'])) {
                    skin::_fallback($this->value[$key]['STYLE'], $element['STYLE'], $skinDir->getPath());
                }
                if (!empty($element['SCRIPT'])) {
                    skin::_fallback($this->value[$key]['SCRIPT'], $element['SCRIPT'], $skinDir->getPath());
                }
                if (!empty($element['LANGUAGE'])) {
                    skin::_fallback($this->value[$key]['LANGUAGE'], $element['LANGUAGE'], false);
                }
            }  /* end foreach */
            unset($key, $element);
        } /* end foreach */
        unset($filename);
    }

    /**
     * get
     *
     * @access  public
     * @param   string  $reference  (optional)
     * @param   string  $key        (optional)
     * @return  array
     */
    function get($reference = "*", $key = "*")
    {
        /* settype to STRING */
        $reference = (string) $reference;
        $key       = (string) $key;

        $REF = array();
        if (empty($key)) {
            $key = "*";
        }
        if (empty($reference)) {
            $reference = "*";
        }
        $key       = strtoupper($key);
        $reference = strtoupper($reference);

        if ($reference == "*") {
            assert('!isset($file); /* cannot redeclare variable $file */');
            foreach ($this->files as $file)
            {
                $REF = Hashtable::merge($REF, $file->get($key));
            }
            unset($file);
            return $REF;
        } else if (!isset($this->files[$reference])) {
            return false;
        } else {
            return $this->files[$reference]->get($key);
        }

    }

    /**
     * get filename
     *
     * This returns the path and name of the template file associated with $key,
     * or bool(false) if there is none.
     *
     * @access  public
     * @param   string  $key
     * @return  string|bool
     */
    function getFile($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } else if (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }

        if (!isset($this->value[$key]['FILE'])) {
            return false;
        } else {
            $filename = $this->value[$key]['FILE'];
        }

        global $YANA;
        
        /* load language files associated with the template */
        $languageList = $this->getLanguage($key);
        assert('!isset($language_file); // Cannot redclare $language_file');
        foreach ($languageList as $language_file)
        {
            $YANA->language->read($language_file);
        }
        unset($language_file);
        
        /* prepare a list of css styles associated with the template */
        $style_list = $this->getStyle($key);
        if (is_array($style_list)) {
            $old_style_list = $YANA->view->document->get('SYSTEM_STYLE');
            if (is_array($old_style_list)) {
                $style_list = array_merge($old_style_list, $style_list);
            }
            $style_list = array_unique($style_list);
            $YANA->view->document->insert('SYSTEM_STYLE', $style_list);
        }

        /* prepare a list of javascript files associated with the template */
        $script_list = $this->getScript($key);
        if (is_array($script_list)) {
            $old_script_list = $YANA->view->document->get('SYSTEM_SCRIPT');
            if (is_array($old_script_list)) {
                $script_list = array_merge($old_script_list, $script_list);
            }
            $YANA->view->document->insert('SYSTEM_SCRIPT', $script_list);
        }
        
        /* return the name of the file */
        return $this->getDirectory().$filename;

    }

    /**
     * getLanguage
     *
     * @access  public
     * @param   string  $key
     * @return  array
     */
    function &getLanguage($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            $result = array();
            return $result;
        } else if (empty($key)) {
            /* nothing to do */
            $result = array();
            return $result;
        } else {
            $key = strtoupper($key);
        }

        if (!isset($this->value[$key])) {
            $result = array();
            return $result;
        }

        $file_definition =& $this->value[$key];
        $languageList   =& $this->value[$key]['LANGUAGE'];

        if (!is_null($languageList)) {
            if (is_array($languageList)) {
                return $languageList;
            } else if (is_string($languageList)) {
                $result = array(&$languageList);
                return $result;
            }
        } else {
            $result = array();
            return $result;
        }
    }

    /**
     * getStyle
     *
     * @access  public
     * @param   string  $key
     * @return  array
     */
    function getStyle($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return array();
        } else if (empty($key)) {
            /* nothing to do */
            return array();
        } else {
            $key = strtoupper($key);
        }

        if (!isset($this->value[$key])) {
            return array();
        }

        $file_definition =& $this->value[$key];
        $style_list      = @$file_definition['STYLE'];

        if (!is_array($style_list)) {
            return array();
        } else {
            return $style_list;
        }
    }

    /**
     * getScript
     *
     * @access  public
     * @param   string  $key
     * @return  array
     */
    function getScript($key)
    {
        /* input check */
        if (!is_string($key)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($key)), E_USER_WARNING);
            return false;
        } else if (empty($key)) {
            /* nothing to do */
            return false;
        } else {
            $key = strtoupper($key);
        }

        $file_definition =& $this->value[$key];
        if (isset($file_definition['SCRIPT'])) {
            $script_list = $file_definition['SCRIPT'];
        } else {
            $script_list = "";
        }

        if (empty($key)) {
            return false;
        } else if (!is_array($script_list)) {
            return array();
        } else {
            return $script_list;
        }
    }

    /**
     * getContent
     *
     * @access  public
     * @return  array  (numeric array of strings)
     */
    function getContent()
    {
        $content = $this->dirStream->getContent();
        assert('is_array($content);');
        if (is_array($content)) {
            return $content;
        } else {
            return array();
        }
    }

    /**
     * getDirectory
     *
     * @access  public
     * @return  string
     */
    function getDirectory()
    {
        assert('is_string($this->directory);');
        return $this->directory;
    }

    /**
     * getInfo
     *
     * @access  public
     * @param   string  $filename
     * @return  array
     */
    function getInfo($filename)
    {
        /* input check */
        if (!is_string($filename)) {
            trigger_error(sprintf(YANA_ERROR_WRONG_ARGUMENT, 1, 'String', gettype($filename)), E_USER_WARNING);
            return false;
        }

        $file = ($this->dirStream->getPath()).$filename;
        if (is_file($file)) {
            $REF = SML::getFile($file, CASE_UPPER);
            assert('isset($REF["SKIN_INFO"]) && is_array($REF["SKIN_INFO"]); /* the resource is not a valid skin info file */');
            if (is_array($REF['SKIN_INFO'])) {
                $REF['SKIN_INFO']['LAST_CHANGE'] = filemtime($file);
                clearstatcache();
                return $REF["SKIN_INFO"];
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     *  _fallback
     *
     * implements fallback behaviour
     *
     * @access  private
     * @param   array|null    &$list
     * @param   array|string  &$new
     * @param   string|bool   $prefix
     * @return  array
     * @ignore
     */
    function _fallback(&$list, &$new, $prefix = "")
    {
        if (!empty($new)) {

            /* settype to ARRAY */
            $new  = (array) $new;
            $list = (array) $list;

            /* overwrite defaults where required */
            foreach (array_keys($new) as $i)
            {
                $var =& $new[$i];
                if (is_string($prefix)) {
                    $var = $prefix.$new[$i];
                }
                /* NOTE: Collisions are treated as follows.
                 * Numeric indices will always be appended, no matter what, while textual indices will get replaced.
                 *
                 * This allows the user to decide wether to create an 'anonymous' association (numbered index)
                 * or a named association (textual index) with a stylesheet.
                 *
                 * So in a derived template the user may decide for himself what element to take and what to drop.
                 *
                 * Also note, that you can drop a named association in a derived template by entering string('null') or boolean('false') as source.
                 */
                if (is_int($i) && ($prefix === false || is_file($var))) {
                    array_push($list, $var);
                } else if ($var == 'null' || empty($var)) {
                    $list[$i] = false;
                } else if ($prefix === false || is_file($var)) {
                    $list[$i] = $var;
                } else {
                    $list[$i] = null;
                    trigger_error("The provided value '${i}' => '${var}' is not a valid file resource.", E_USER_WARNING);
                }
            }
        } else {
            /* intentionally left blank */
        }
    }

    /**
     * self-diagnosis
     *
     * implement it this way:
     * $errors = $YANA->skins->check();
     *
     * if (empty($errors)) {
     * print 'all fine';
     * } else {
     * print 'The following errors were reported:'.$errors;
     * }
     *
     * @access  public
     * @return  string
     * @ignore
     */
    function check()
    {
        /* String */ $txt = "";
        global $YANA;

        foreach ($this->value as $key => $element)
        {
            $filename = $this->getFile($key);
            if (!empty($filename) && !file_exists($filename)) {
                $txt .= "<p>ERROR: Skin entry '${key}' references to file '${filename}'. But this file does not exist.<br>Please make sure this path and filename is correct and you have all files installed. Reinstall this skin if necessary.</p>\n";
            }
            /* check language references */
            if (!empty($element['LANGUAGE']) && is_array($element['LANGUAGE']) && count($element['LANGUAGE']) > 0) {
                assert('!isset($value); /* cannot redeclare variable $value */');
                foreach ($element['LANGUAGE'] as $value)
                {
                    if (!empty($value)) {
                        if (!$YANA->language->read($value)) {
                            $txt .= "<p>ERROR: A required language file '${value}.config' for template '${key}' is not available.<br>This template may not be displayed correctly. Please check if the chosen language file is correct and update your language pack if needed.</p>\n";
                        }
                    }
                }
                unset($value);
            }
            /* check stylesheet references */
            if (!empty($element['STYLE']) && is_array($element['STYLE']) && count($element['STYLE']) > 0) {
                assert('!isset($value); /* cannot redeclare variable $value */');
                foreach ($element['STYLE'] as $value)
                {
                    if (is_null($value)) {
                        $txt .= "<p>ERROR: A required stylesheet for template '${key}' is not available.<br>This template may not be displayed correctly.</p>\n";
                    }
                }
                unset($value);
            }
            /* check script references */
            if (!empty($element['SCRIPT']) && is_array($element['SCRIPT']) && count($element['SCRIPT']) > 0) {
                assert('!isset($value); /* cannot redeclare variable $value */');
                foreach ($element['SCRIPT'] as $value)
                {
                    if (is_null($value)) {
                        $txt .= "<p>ERROR: A required javascript file for template '${key}' is not available.<br>This template may not be displayed correctly.</p>\n";
                    }
                }
                unset($value);
            }
        } /* end foreach */
        return $txt;
    }

}

?>