<?php
/**
 * YANA library
 *
 * @package yana
 * @subpackage vdrive
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * VDrive_dir
 *
 * class representing virtual directories
 *
 * @access     public
 * @package    yana
 * @subpackage vdrive
 *
 * @ignore
 */
class VDrive_dir extends VDrive_mountpoint {

    /**
     * Note: this overwrites the variable of the parent class
     *
     * @access  private
     * @var     string
     * @ignore
     */
    var $type = "dir";

    /**
     * constructor
     *
     * @param  array   $SRC
     * @param  string  $type
     * @param  bool    $automount
     * @param  string  $filter
     */
    function VDrive_dir($SRC, $type, $automount, $filter = "")
    {
        /* settype to ARRAY
         *            STRING
         *            BOOLEAN
         *            STRING
         */
        $SRC       = (array)  $SRC;
        $type      = (string) $type;
        $automount = (bool)   $automount;
        $filter    = (string) $filter;

        switch ($type) {
            case "dir":
                $this->mountpoint = new dirStream($SRC[0]);
                $this->mountpoint->filter = $filter;
                if($automount) {
                    $this->mount();
                }
            break;
            default:
                trigger_error("Invalid directory-type argument '$type' supplied for Mountpoint '".print_r($SRC[0],true)."'.", E_USER_WARNING);
            break;
        }

    }

    /**
     * mount the current resource
     *
     * Returns bool(true) on success and bool(false) on error.
     * In this case "mounting" means it calls the function "read()"
     * on the resource object, if it has it.
     *
     * Note: this overwrites the function of the parent class
     *
     * @access public
     * @return boolean
     */
    function mount()
    {
        assert('is_bool($this->is_mounted);');
        if ($this->isMounted()) {
            return true;
        } else {
            if (is_object($this->mountpoint) && method_exists($this->mountpoint, "read")) {
                $this->mountpoint->read();
                $this->is_mounted = true;
                return true;
            } else {
                return false;
            }
        }
    }

}

?>
