<?php
/**
 * Handling of Config- and SML-Files
 *
 * @package     yana
 * @subpackage  utilities
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * @deprecated since 2.8.5
 */

if (!defined('CASE_MIXED')) {
    /**
     * @ignore
     */
    define('CASE_MIXED', -1);
}

/**
 * Read a config file and copy its contents to the global scope
 *
 * The argument $mode can be used to decide how keys should be treated.
 *
 * Valid values for $mode are:
 * <ul>
 *     <li>  CASE_UPPER  upper-case all keys       </li>
 *     <li>  CASE_LOWER  lower-case all keys       </li>
 *     <li>  CASE_MIXED  leave keys in mixed case  </li>
 * </ul>
 *
 * Returns bool(true) on success and bool(false) on error.
 *
 * @name    importConfigFile()
 * @param   string  $filename
 * @param   int     $mode
 * @return  bool
 *
 * @see    getConfigFile()
 * @see    makeConfig()
 *
 * @deprecated since 2.8.5
 */
function importConfigFile($filename, $mode = CASE_UPPER)
{
    /**
     * Input checking
     */
    assert('!isset($_chk);');
    assert('!isset($_types);');
    $_types = array(
    /* argument 1 */ IS_STRING,
    /* argument 2 */ IS_INT
    );
    $_chk = checkArgumentList(func_get_args(), $_types, __FUNCTION__);
    if ($_chk !== true) {
        trigger_error($_chk, E_USER_WARNING);
        return false;
    } else {
        unset($_chk, $_types);
    }

    assert('$mode === CASE_MIXED || $mode === CASE_LOWER || $mode === CASE_UPPER;');
    $daten =& getConfigFile($filename, $mode);
    foreach ($daten as $key => $element)
    {
        if (isset($GLOBALS[$key])) {
            $GLOBALS[$key] += $element;
        } else {
            $GLOBALS[$key]  = $element;
        }
    }
    return true;
}

/**
 * Read variables from an encoded string
 *
 * This function has been superceeded by the OO-style
 * implementation SML::decode() and since version 2.8.5
 * has been marked deprecated.
 *
 * @name   decodeConfigString()
 * @param  string    $input
 * @param  int       $mode
 * @return array
 *
 * @see    importConfigFile()
 * @see    getConfigFile()
 * @see    makeConfig()
 *
 * @deprecated since 2.8.5
 */
function decodeConfigString($input, $mode = CASE_UPPER)
{
    return SML::decode($input, $mode);
}

/**
 * Read a config file and return its contents
 *
 * This function has been superceeded by the OO-style
 * implementation SML::getFile() and since version 2.8.5
 * has been marked deprecated.
 *
 * @name   getConfigFile()
 * @param  array|string  $input
 * @param  int           $mode
 * @return array
 *
 * @see    importConfigFile()
 * @see    makeConfig()
 *
 * @deprecated since 2.8.5
 */
function getConfigFile($input, $mode = CASE_UPPER)
{
    return SML::getFile($input, $mode);
}

/**
 * Create a config file from scalar variable or array data.
 *
 * This function has been superceeded by the OO-style
 * implementation SML::encode() and since version 2.8.5
 * has been marked deprecated.
 *
 * @name   makeConfig()
 * @param  scalar|array  $data
 * @param  string        $name
 * @param  int           $mode
 *
 * @see    getConfigFile()
 * @see    importConfigFile()
 *
 * @deprecated since 2.8.5
 */
function makeConfig($data, $name = null, $mode = CASE_UPPER)
{
    return SML::encode($data, $name, $mode);
}
?>