<?php
/**
 * default library
 * 
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * Default library for common functions
 *
 * This plugin is important. It provides functionality
 * that might be usefull for other plugins.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_default_library extends Plugin
{

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        /* check input data */
        assert('is_string($event);');
        assert('is_array($ARGS);');
        settype($event, "string");
        settype($ARGS, "array");
        $event = strtolower($event);
        global $YANA;

        /* BEGIN visitor-counter */
        $my_counter = new counter();
        if (empty($ARGS['id'])) {
            $id = 'default';
        } else {
            $id = $ARGS['id'];
        } 
        $YANA->setVar('BESUCHER', $my_counter->count($id));
        /* END visitor-counter */

        return true;
    }

    /**
     * Create graphical visitor counter
     *
     * Note: This function ends the program.
     *
     * parameters taken:
     *
     * <ul>
     * <li> string id </li>
     * <li> string target  </li>
     * </ul>
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function graphic_counter ($ARGS)
    {
        global $YANA;
        if (!empty($ARGS['id'])) {
            $my_counter = new counter();
            $count = $my_counter->count($ARGS['id']);
        } else {
            $my_counter = new counter();
            $count = $my_counter->count('default');
        }

        settype($ARGS['target'], 'int');

        $skinDir = $YANA->getVar("SKINDIR")."default/counter/".$ARGS['target']."/";
        if (!is_dir($skinDir)) {
            $skinDir = $YANA->getVar("SKINDIR")."default/counter/0/";
        }
        $myImage = new Image($skinDir."hintergrund.jpg", 'jpg');
        $myImageValues = Image::getSize($skinDir."hintergrund.jpg");
        $previousImageWidths = 0;
        
        if (file_exists($skinDir."blank.png")) {
            $imageValues = Image::getSize($skinDir."blank.png");
            $previousImageWidths += $imageValues[0];
        }
        
        for ($i = 0; $i < strlen("$count"); $i++)
        {

            $filename = $skinDir.substr("$count", strlen("$count")-$i-1, 1).".png";
            $imageValues = Image::getSize($filename);
            $previousImageWidths += $imageValues[0];
            if (($myImageValues[0]-$previousImageWidths) < 0 || ($myImageValues[1]-$imageValues[1]) < 0) {
               unset($myImage);
               $myImage = new Image();
               $myImage->resize(325,55);
               $myImage->drawString("[ERROR] image size exceeds maximum",                                              5,  0);
               $myImage->drawString("or value to big to be displayed",                                                 5, 10);
               $myImage->drawString("1) check size of counter \"".substr("$count", strlen("$count")-$i-1, 1).".png\"", 5, 20);
               $myImage->drawString("2) check dimensions of file \"hintergrund.png\"",                                 5, 30);
               $myImage->drawString("   are big enough for the content.",                                              5, 40);
               $myImage->outputToScreen();
               die;
            };
            $myImage->copyRegion(new Image($filename, 'png'), 0, 0, null, null, ($myImageValues[0]-$previousImageWidths), (int) floor(($myImageValues[1]-$imageValues[1])/2));
            unset($image);

        }

        $myImage->outputToScreen();
        die();
    }

    /**
     * Show Sitemap
     *
     * this function does not expect any arguments
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function sitemap ($ARGS)
    {
        /* this function expects no arguments */

        global $YANA;
        $id = $YANA->getVar('ID');
        if (!empty($id)) {
            $my_counter = new counter();
            $YANA->setVar('BESUCHER', $my_counter->count($id));
        } else {
            $my_counter = new counter();
            $YANA->setVar('BESUCHER', $my_counter->count('default'));
        }
        $YANA->setVar("PROFILES", $YANA->plugins->getPluginList());

        return true;
    }

    /**
     * Clear Server Template-Cache
     *
     * this function does not expect any arguments
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function clear_server_cache ($ARGS)
    {
        /* this function expects no arguments */
        smartTemplate::clearCache();
        return true;
    }

    /**
     * Create preview message
     *
     * Note: this function ends the program
     *
     * parameters taken:
     *
     * <ul>
     * <li> string eintraege    text to be previewed </li>
     * </ul>
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function preview ($ARGS)
    {
        global $YANA;
        if (!isset($ARGS['eintraege'])) {
            trigger_error("Required argument 'eintraege' has not been given.", E_USER_ERROR);
            die();
        } else if (!is_string($ARGS['eintraege'])) {
            trigger_error("Wrong argument type for argument 'eintraege'. String expected, found ".gettype($ARGS['eintraege'])." instead.", E_USER_ERROR);
            die();
        } else {
            $filename = $YANA->skin->getFile('blank');
            assert('!empty($filename);');
            $blankDoc = new smartTemplate($filename);
            $blankDoc->insert('*',$YANA->getVar("*"));
            assert('is_string($ARGS["eintraege"]);');
            $ARGS['eintraege'] = untaintInput($ARGS['eintraege'], "text", 0, YANA_ESCAPE_USERTEXT);
            $ARGS['eintraege'] = DisplayUtility::smilies($ARGS['eintraege']);
            $ARGS['eintraege'] = DisplayUtility::embeddedTags($ARGS['eintraege']);
            $blankDoc->insert('*','<div style="height: 100%; overflow: auto;">'.$ARGS['eintraege'].'</div>');
            die($blankDoc->get());
        }
    }

    /**
     * Create CAPTCHA-image
     *
     * parameters taken:
     *
     * <ul>
     * <li> int security_image_index    index of image to display </li>
     * </ul>
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function security_get_image ($ARGS)
    {
        global $YANA;
        $imagesrc = $YANA->getVar("CONFIGDIR")."security_image/security_image".rand(0,9).".png";
        $file     = new datfile($YANA->getVar("CONFIGDIR")."security_image/security_image.dat");
       
        if (!$file->exists()) {
            $file->create();
        }
        $file->read();
        if (!$file->isEmpty()) {
            $contents = $file->get(0);
        } else {
            $contents = array();
        }

        if (!isset($contents['TIME']) || $contents['TIME'] < time() || $contents['MAX_TIME'] < time()) {
            $contents = array();
            $contents['MAX_TIME'] = time() + 10000;
            for ($i=1;$i<10;$i++)
            {
                $contents["_$i"] = "";
                for ($j=0;$j<5;$j++)
                {
                    $letter = "";
                    while (empty($letter) || strtolower($letter) == 'o' || $letter == '0' || $letter == 'l' || $letter == 'i')
                    {
                        switch (rand(0,2))
                        {
                            case 0:
                                $letter = chr(rand(65,90));
                            break;
                            case 1:
                                $letter = chr(rand(48,57));
                            break;
                            case 2:
                                $letter = chr(rand(97,122));
                            break;
                        }
                    }
                    $contents["_$i"] .= $letter;
                }
            }
        }
        $contents['TIME'] = time() + 1200;
        $file->remove(0);
        $file->insert($contents);
        $file->write();

        if (!isset($ARGS['security_image_index']) || $ARGS['security_image_index'] < 1 || $ARGS['security_image_index'] > 9) {
            $text =& $contents['_1'];
        } else {
            $text =& $contents['_'.$ARGS['security_image_index']];
        }

        $image = new Image($imagesrc, 'png');
        for ($i = 0; $i < 5; $i++)
        {
            $image->drawString($text[$i], 4+($i*9)+rand(0,1), 1+rand(-1,1), $image->getColor(40+rand(-30,60),40+rand(-30,60),40+rand(-30,60)), 5);
        }
        $image->outputToScreen();
        die();
    }

    /**
     * Test if a string matches the corresponding CAPTCHA
     *
     * parameters taken:
     *
     * <ul>
     * <li> int security_image_index    index of image to display </li>
     * <li> string security_image       text to compare with CAPTCHA </li>
     * </ul>
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function security_check_image ($ARGS)
    {
        global $YANA;
        $file = new datfile($YANA->getVar("CONFIGDIR")."security_image/security_image.dat");
        $file->read();
        if (!$file->isEmpty()) {
            $contents = $file->get(0);
        } else {
            $contents = array();
        }

        if ($contents['MAX_TIME'] < time()) {
            return false;
        } else {
            if (!isset($ARGS['security_image_index']) || $ARGS['security_image_index'] < 1 || $ARGS['security_image_index'] > 9) {
                $text =& $contents['_1'];
            } else {
                $text =& $contents['_'.$ARGS['security_image_index']];
            }
            if (!empty($text) && ($ARGS['security_image'] == $text)) {
                return true;
            } else {
                return false;
            }
        }
    }

    /**
     * Create a microsummary for a page
     *
     * Technical notes:
     *
     * parameters taken:
     *
     * <ul>
     * <li> string id    identifies summary to get </li>
     * </ul>
     *
     * Note: this function terminates program execution.
     *
     * Semantic description:
     *
     * "Microsummaries" are a Firefox 2.0 feature that allows users
     * to create dynamic bookmark titles that automatically updates
     * when the content of the bookmarked page changes.
     *
     * Have a look at what microsummaries can be:
     * <ul>
     *   <li> the numbers of downloads of a file on a download site </li>
     *   <li> the latest news on a news page </li>
     *   <li> current number of unread e-mail in the inbox of a webmail service </li>
     *   <li> current total of donations to a project </li>
     *   <li> the date of latest updates on a database </li>
     *   <li> the latest submission to a guestbook or forum </li>
     *   <li> the number of visitors currently online in a chat room </li>
     *   <li> the latest stock values aso. </li>
     * </ul>
     *
     * Examples of usage:
     * <ol>
     *  <li> Setting a microsummary from a plugin:
     *       <code>Microsummary::set($this->name, 'Summary text');</code>
     *  </li>
     *  <li> Retrieving a microsummary in a plugin:
     *       <code>$microsummary = Microsummary::get($this->name);</code>
     *  </li>
     *  <li> To indicate that a microsummary exists for your plugin
     *       add this as the last line in your plugin constructor
     *       <code>Microsummary::publish($this->name);</code>
     *  </li>
     *  <li> Calling a microsummary from a browser:
     *       <code>index.php?action=get_microsummary&target=guestbook</code>
     *       (where 'guestbook' is the name of the plugin)
     *  </li>
     * </ol>
     *
     * @access public
     * @param array $ARGS array of params passed to the function
     * @return boolean
     */
    function get_microsummary ($ARGS)
    {
        if (empty($ARGS['target']) || !is_string($ARGS['target'])) {
            print 'Error: illegal request';
            exit (1);
        } else {
            $microsummary = Microsummary::get($ARGS['target']);
            if (!empty($microsummary)) {
                print $microsummary;
                exit(0);
            } else {
                print 'No summary available';
                exit(1);
            }
        }
    }

    /**
     * Create an image or download a file from a source
     *
     * called by GUI-generator
     *
     * @access  public
     * @param   array  $ARGS  array of params passed to the function
     * @return  bool
     * @ignore
     */
    function download_file ($ARGS)
    {
        global $YANA;

        if (!isset($ARGS['target'])) {
            return false;
        } else {
            $i = (int) $ARGS['target'];
            if (!isset($_SESSION['filelist'][$i])) {
                return false;
            } else {
                $source = new String($_SESSION['filelist'][$i]);
                $source->encrypt('xor', YANA_INSTANCE_ID.session_id());
                if ($source->match('/^(?:thumb|image|file)\.\w+\.(?:png|jpg|jpeg|gif|gz)$/s') === false) {
                    return false;
                }
                if (!empty($ARGS['fullsize'])) {
                    $source->replace('thumb.', 'image.');
                } else {
                    $source->replace('image.', 'thumb.');
                }
                $source = $YANA->getVar('DBBLOB') . $source->get();
            }
        }

        if (!is_file($source)) {
            /* error - file does not exist */
            return false;
        } else {

            /* download a file */
            if (preg_match('/\.gz$/', $source)) {

                $i = 0;

                $gz = gzopen($source, 'r');
                while (!gzeof($gz))
                {
                    $buffer = gzgets($gz, 4096);
                    switch ($i)
                    {
                        case 0:
                            if (preg_match('/^[\w\.\d\-\_]+$/s', $buffer) && !headers_sent()) {
                                header("Content-Disposition: inline; filename=${buffer}");
                            }
                        break;
                        
                        case 1:
                            if (is_numeric($buffer) && !headers_sent()) {
                                header("Content-Length: ${buffer}");
                            }
                        break;

                        case 2:
                            if (preg_match('/^\w+\/\w+$/s', $buffer) && !headers_sent()) {
                                header("Content-type: ${buffer}");
                            }
                        break;
                        
                        default:
                            print $buffer;
                        break;

                    }
                    $i++;
                }
                gzclose($gz);

            /* download an image */
            } else {
                $image = new Image($source);
                $image->outputToScreen();
            }
            exit(0);
        }
    }

}

//  2007, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>