<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * guestbook plugin
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_guestbook extends Plugin
{

    /**#@+
     * @access  private
     * @ignore
     */

    /** @var mixed             */ var $database             = null;
    /** @var string (readonly) */ var $ACTION_SHOW          = "guestbook_read_read";
    /** @var string (readonly) */ var $ACTION_RSS           = "guestbook_read_rss";
    /** @var string (readonly) */ var $ACTION_DETAILS       = "guestbook_read_read_seperated";
    /** @var string (readonly) */ var $ACTION_NEW           = "guestbook_default_new";
    /** @var string (readonly) */ var $ACTION_NEW_WRITE     = "guestbook_write_new";
    /** @var string (readonly) */ var $ACTION_DELETE        = "guestbook_write_delete";
    /** @var string (readonly) */ var $ACTION_EDIT          = "guestbook_read_edit";
    /** @var string (readonly) */ var $ACTION_EDIT_WRITE    = "guestbook_write_edit";
    /** @var string (readonly) */ var $ACTION_COMMENT       = "guestbook_default_comment";
    /** @var string (readonly) */ var $ACTION_COMMENT_WRITE = "guestbook_write_comment";

    /**#@-*/

    /**
     * Constructor
     *
     * @access public
     * @param  string  $name  a name for this plugin
     * @ignore
     */
    function plugin_guestbook($name)
    {
        /* call superclass constructor */
        $constructor = get_parent_class(__CLASS__);
        parent::$constructor($name);

        global $YANA;
        $this->database = $YANA->connect('guestbook');

        /* Plugin-specific addities */
        $YANA->setVar("ACTION_SHOW"         , $this->ACTION_SHOW);
        $YANA->setVar("ACTION_RSS"          , $this->ACTION_RSS);
        $YANA->setVar("ACTION_DETAILS"      , $this->ACTION_DETAILS);
        $YANA->setVar("ACTION_NEW"          , $this->ACTION_NEW);
        $YANA->setVar("ACTION_NEW_WRITE"    , $this->ACTION_NEW_WRITE);
        $YANA->setVar("ACTION_DELETE"       , $this->ACTION_DELETE);
        $YANA->setVar("ACTION_EDIT"         , $this->ACTION_EDIT);
        $YANA->setVar("ACTION_EDIT_WRITE"   , $this->ACTION_EDIT_WRITE);
        $YANA->setVar("ACTION_COMMENT"      , $this->ACTION_COMMENT);
        $YANA->setVar("ACTION_COMMENT_WRITE", $this->ACTION_COMMENT_WRITE);
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        /* check input data */
        settype($event, "string");
        settype($ARGS, "array");
        $event = strtolower($event);

        global $YANA;

        switch ($event)
        {

            /* Type: write, Permission: 75, Templates: MESSAGE  */
            case $this->ACTION_EDIT_WRITE:
                /* convert raw form data into arrays */
                $ENTRIES = array();
                if (empty($ARGS['selected_entries']) || !is_array($ARGS['selected_entries'])) {
                    $YANA->report(new Alert("ungueltige_eingabe"));
                    return false;
                } else {

                    $ENTRIES = GuiCreator::getFormdata(GUI_EDIT, $this->database->structure, "guestbook");

                    /* loop through selected entries */
                    foreach ($ENTRIES as $id => $entry)
                    {
                        $id = strtolower($id);
            
                        /* before doing anything, check if entry exists */
                        if (!$this->database->exists("guestbook.${id}")) {
            
                            /* error - no such entry */
                            $YANA->report(new Alert('ungueltige_eingabe'));
                            $YANA->report(new Log("The selected entry guestbook.$id does not exist!"));
                            return false;
            
                        /* update the row */
                        } else {
                            
                            /* 1) If dataset is empty, skip the value and proceed */
                            if (empty($entry)) {
                                continue;

                            /**
                             * 2) If the update operation was not successful, issue an error
                             *    message and abort. (will also forfeit all previously made,
                             *    uncommited changes)
                             */
                            } else if (!$this->database->insert("guestbook.${id}", $entry)) {
                                $YANA->report(new Alert("ungueltige_eingabe"));
                                return false;

                            /**
                             * 3) If all was fine, proceed to next value.
                             */
                            } else {
                                continue;
                            }

                        }
                    } /* end for */

                    /* don't forget to save your recent changes ;-) */
                    if ($this->database->write()) {
                        return true;
                    } else {
                        return false;
                    }
                }
            break;

            /* Type: write, Permission: 75, Templates: MESSAGE  */
            case $this->ACTION_DELETE:
                /* find any previously marked entries and mark them for removal */
                $ENTRIES = array();
                /* loop through selected entries */
                foreach($ARGS['selected_entries'] as $id)
                {
                    $id = strtolower($id);

                    /**
                     * 1) If the entry to delete does not exist,
                     *    issue an error message and abort. (will also forfeit all
                     *    previously made, uncommited changes)
                     */
                    if (!$this->database->exists("guestbook.$id")) {
                        $YANA->report(new Alert("ungueltige_eingabe"));
                        $YANA->report(new Log("The selected entry guestbook.$id does not exist!"));
                        return false;

                    }

                    /**
                     * 2) If the delete operation was not successful, issue an error
                     *    message and abort. (will also forfeit all previously made,
                     *    uncommited changes)
                     */
                    if (!$this->database->remove("guestbook.$id")) {
                        $YANA->report(new alert("ungueltige_eingabe"));
                        return false;

                    /**
                     * 3) If all was fine, proceed to next value.
                     */
                    } else {
                        continue;
                    }
                } /* end foreach */
                /* now delete those entries */
                if ($this->database->write()) {
                    return true;
                } else {
                    return false;
                }
            break;

            /* Type: default, Permission: 0, Templates:  GUESTBOOK_NEW */
            case $this->ACTION_NEW:
                $YANA->setVar('RSS_ACTION',$this->ACTION_RSS);
                if ($this->_securityCheck() === false) {
                    return false;
                }
                $YANA->setVar("SECURITY_IMAGE_INDEX", rand(1,9));
                $YANA->setVar("BESCHREIBUNG", '%LANGUAGE.DESCR_NEW%');
                $YANA->setVar('RSS_ACTION',$this->ACTION_RSS);
                return true;
            break;

            /* Type: read, Permission: 0, Templates:  NULL */
            case $this->ACTION_RSS:
                if (!class_exists('RSS')) {
                    trigger_error("RSS wrapper class not found. Unable to proceed.", E_USER_ERROR);
                    return false;
                } else {
                    if ($this->_securityCheck() === false) {
                        return false;
                    }

                    /* get entries */
                    $args = $this->_createArguments(array());
                    assert('is_array($args); /* unexpected result: $args */');                    
                    $rows = $this->_getTable($args, "guestbook");
                    assert('is_array($rows); /* unexpected result: $rows */');

                    /* create RSS feed */
                    $rss = new RSS();
                    $YANA->language->read('guestbook');
                    $rss->description = $YANA->getVar('LANGUAGE.RSS_DESCRIPTION');
                    if (empty($rss->description)) {
                        $rss->description = 'the 10 most recent guestbook entries';
                    }
                    DisplayUtility::loadSmilies();
                    foreach ($rows as $row)
                    {
                        $item = new RSSitem();
                        /* 1) process title */
                        $item->title = $row['GUESTBOOK_NAME'];
                        /* 2) process link */
                        $id = $row['GUESTBOOK_ID'];
                        $item->link = DisplayUtility::url('action='.$this->ACTION_SHOW.'&target='.$id, true);
                        /**
                         * {@internal
                         * Note: guid is ignored by some aggregators and link is used instead
                         * where this is the case, the session id can not be part of the uri as this
                         * is subject to change. }}
                         */
                        $item->link = str_replace(session_name()."=".session_id(),'',$item->link);
                        /* 3) process description */
                        $item->description = $row['GUESTBOOK_MESSAGE'];
                        $item->description = DisplayUtility::embeddedTags($item->description);
                        $item->description = DisplayUtility::smilies($item->description);
                        $item->description = strip_tags($item->description);
                        if (strlen($item->description) > 500) {
                            $item->description = substr($item->description,0,496).' ...';
                        }
                        /* 4) process pubDate */
                        $item->pubDate     = $row['GUESTBOOK_DATE'];
                        if (is_numeric($item->pubDate)) {
                            $item->pubDate = date('r', $item->pubDate);
                        }
                        $rss->addItem($item);
                    } /* end foreach */
                    print utf8_encode($rss->toString());
                    exit(0);
                    return true;
                }
            break;

            /* Type: write, Permission: 0, Templates: MESSAGE  */
            case $this->ACTION_NEW_WRITE:
                if ($this->_securityCheck() === false) {
                    return false;
                }

                $permission = $YANA->getVar("PERMISSION");
                /* avoid spamming */
                if (!is_int($permission) || $permission < 1) {
                    if ($YANA->getVar("PROFILE.GUESTBOOK.SPAMPROTECT") === true && !$YANA->handle("security_check_image", $ARGS)) {
                        $YANA->report(new alert('ungueltige_eingabe', 'Security check failed!'));
                        $YANA->report(new log('SPAM: CAPTCHA not solved, entry has not been created.'));
                        return false;
                    }
                }

                /* get values from form fields to create new entry */
                $NEW_ENTRY = GuiCreator::getFormdata(GUI_NEW, $this->database->structure, "guestbook");
                $NEW_ENTRY['profile_id'] = $YANA->session->getId();

                plugin_guestbook::_untaintInput($NEW_ENTRY);
                if (empty($NEW_ENTRY)) {
                    return false;
                }

                /* mark registered users */
                if (strcasecmp($YANA->getVar("SESSION_USER_ID"), $ARGS['guestbook_name']) == 0) {
                    $NEW_ENTRY['guestbook_is_registered'] = 1;
                }

                /* avoid flooding */
                $myFlood = $YANA->plugins->get($this->name, "flood");
                $myFlood->setMax((int)$YANA->getVar("PROFILE.GUESTBOOK.FLOODING"));
                if ($myFlood->isBlocked()) {
                    $YANA->report(new Log('Possibly flooding attempt detected. User request rejected.'));
                    $YANA->report(new Alert('flooding'));
                    return false;
                }
                $recent_entry = $this->database->get("guestbook.?.guestbook_message", array(array('profile_id', $YANA->session->getId(), '=')));
                if(!empty($NEW_ENTRY['guestbook_message']) && $recent_entry == $NEW_ENTRY['guestbook_message']) {
                    $YANA->report(new Alert('nospam'));
                    return false;
                } elseif($myFlood->getMax() > 0) {
                    $myFlood->set();
                }

                /* insert new entry into table */
                if (!$this->database->insert('guestbook.*', $NEW_ENTRY)) {
                    $YANA->report(new Log('Failed to insert entry.', preg_replace('/\s/', ' ', SML::encode($NEW_ENTRY)) ));
                    $YANA->report(new Alert('ungueltige_eingabe'));
                    return false;
                } elseif (!$this->database->write()) {
                    $YANA->report(new Log('Unable to submit entry.', preg_replace('/\s/', ' ', SML::encode($NEW_ENTRY)) ));
                    return false;
                } else {
                    /* send Mail */
                    if (!empty($NEW_ENTRY['guestbook_mail']) && $YANA->getVar("PROFILE.GUESTBOOK.SENDMAIL")) {
                        $this->_sendMail($NEW_ENTRY['guestbook_mail'], $YANA->skin->getFile("danke_mail"), $ARGS);
                    }
                    if ($YANA->getVar("PROFILE.GUESTBOOK.MAIL") && $YANA->getVar("PROFILE.GUESTBOOK.NOTIFICATION")) {
                        $this->_sendMail($YANA->getVar("PROFILE.GUESTBOOK.MAIL"),$YANA->skin->getFile("benachrichtigung_mail"),$ARGS);
                    }
                    if (class_exists('Microsummary')) {
                        Microsummary::set($this->name, strtoupper($this->name) . ', update '.date('d M y G:s', time()));
                    }
                    return true;
                }
            break;

            /* Type: read, Permission: 0, Templates:  GUESTBOOK_READ */
            case $this->ACTION_SHOW:
                if ($this->_securityCheck() === false) {
                    return false;
                }

                /* get entries */
                if (isset($ARGS['target']) && is_numeric($ARGS['target'])) {
                    $args = $this->_createArguments($ARGS);
                    $id = (int) $ARGS['target'];
                    $args['where'][] = array("guestbook_id", $id, "=");
                } else {
                    $args = $this->_createArguments($ARGS);
                }
                assert('is_array($args); /* unexpected result: $args */');
                $rows = $this->_getTable($args, "guestbook");
                assert('is_array($rows); /* unexpected result: $rows */');

                $YANA->setVar('DATABASE', Hashtable::changeCase($this->database->structure->getStructure(), CASE_UPPER));
                $YANA->setVar('ROWS', $rows);
                $YANA->setVar("BESCHREIBUNG", '%LANGUAGE.DESCR_SHOW%');
                $YANA->setVar('RSS_ACTION', $this->ACTION_RSS);
                return true;
            break;


            /* Type: read, Permission: 0, Templates:  GUESTBOOK_READ_SEPERATED */
            case $this->ACTION_DETAILS:
                if ($this->_securityCheck() === false) {
                    return false;
                }

                /* get entries */
                $args = $this->_createArguments($ARGS);
                assert('is_array($args); /* unexpected result: $args */');
                $rows = $this->_getTable($args, "guestbook");
                assert('is_array($rows); /* unexpected result: $rows */');


                $YANA->setVar('DATABASE', Hashtable::changeCase($this->database->structure->getStructure(), CASE_UPPER));
                $YANA->setVar('ROWS', $rows);
                $YANA->setVar("BESCHREIBUNG", '%LANGUAGE.DESCR_DETAILS%');
                $YANA->setVar('RSS_ACTION', $this->ACTION_RSS);
                return true;
            break;

            /* Type: default, Permission: 30, Templates: GUESTBOOK_DEFAULT_COMMENT  */
            case $this->ACTION_COMMENT:
                if (!isset($ARGS['target']) || !is_numeric($ARGS['target'])) {
                    $YANA->report( new Alert('ungueltige_eingabe'));
                    $YANA->exitTo($this->ACTION_SHOW);
                    return false;
                } else {
                    settype($ARGS['target'], 'int');
                    $guestbook_comment = $this->database->get('guestbook.'.$ARGS['target'].'.guestbook_comment');
                    if (!empty($guestbook_comment)) {
                         $YANA->setVar('GUESTBOOK_COMMENT', $guestbook_comment);
                    }
                    return true;
                }
            break;


            /* Type: write, Permission: 30, Templates: MESSAGE  */
            case $this->ACTION_COMMENT_WRITE:

                /**
                 * 1) error: invalid input
                 */
                if (!isset($ARGS['target']) || !is_numeric($ARGS['target']) || !is_string($ARGS['guestbook_comment'])) {
                    $YANA->report( new Alert('ungueltige_eingabe'));
                    $YANA->exitTo($this->ACTION_COMMENT);
                    return false;

                } else {
                    $id      = strtolower($ARGS['target']);
                    $comment = $ARGS['guestbook_comment'];
                }

                /**
                 * 2) If the update operation was not successful, issue an error
                 *    message and abort.
                 */
                if (!$this->database->insert("guestbook.${id}.guestbook_comment", $comment)) {
                    $YANA->report(new log("Unable to insert comment at 'guestbook.${id}.'", array('guestbook_comment' => $comment)));
                    return false;
                } else if (!$this->database->write()) {
                    $YANA->report(new log("Unable to commit changes to entry 'guestbook.${id}.'", array('guestbook_comment' => $comment)));
                    return false;

                /**
                 * 3) If all was fine, return true.
                 */
                } else {
                    return true;
                }
            break;

            default:
                /* intentionally left blank */
            break;

        } /* end switch */

        return true;
    }

    /**
     * _getTable
     *
     * @access  private
     * @param   array   $ARGS
     * @param   string  $table
     * @return  array
     * @ignore
     */
    function _getTable($ARGS, $table)
    {
        global $YANA;

        /* optimistic type casting */
        @settype($table, "string");
        @settype($ARGS['sort'], "string");
        @settype($ARGS['page'], "integer");
        @settype($ARGS['entries'], "integer");
        @settype($ARGS['desc'], "boolean");

        /* check if $table really is a table */
        if (!$this->database->structure->isTable($table)) {
            trigger_error("The provided argument '${table}' is not a registered table.", E_USER_WARNING);
            return array();
        } else {
            $table = strtolower($table);
        }
        
        /* check if a $where clause is provided
         *
         * The where-clause will be checked for correct syntax later.
         * No need to do this here.
         * (esp. as this argument comes from inside the script anyway and contains no user input).
         */
        if (empty($ARGS['where']) || (!is_string($ARGS['where']) && !is_array($ARGS['where']))) {
            $where = "";
        } else {
            $where = $ARGS['where'];
        }

        /* get the name of the primary key */
        $primary_key = $this->database->structure->getPrimaryKey($table);
        assert('is_string($primary_key); /* unexpected result: $primary_key */');
        
        /* check if sorting order is $desc */
        if (isset($ARGS['desc'])) {
            if ($ARGS['desc'] === true) {
                $desc = true;
            } else {
                $desc = false;
            }
        } else {
            $desc = true;
        }
        
        /* determine the start page */
        if ($ARGS['page'] < 0) {
            $page = 0;
        } else { 
            $page = (int) $ARGS['page'];
        }

        /* determine the limit argument */
        $entPerPage = (int) $ARGS['entries'];
        if ($entPerPage < 1) {
            $entPerPage = (int) $YANA->getVar("PROFILE.GUESTBOOK.ENTPERPAGE");
        }
        if ($entPerPage < 1) {
            trigger_error("The registry key PROFILE.GUESTBOOK.ENTPERPAGE contains an invalid value. Thus the default '10' will be used instead.", E_USER_NOTICE);
            $entPerPage = 10;
        }

        /* determine the limit argument */
        if (!empty($ARGS['sort']) && !$this->database->structure->isColumn($table, $ARGS['sort'])) {
            $sortBy = $ARGS['sort'];
        } else {
            $sortBy = $primary_key;
        }

        /* get rows from database */
        $result = $this->database->get($table, $where, $sortBy, $page, $entPerPage, $desc);

        /* set template vars */

        $YANA->setVar('TABLE', $table); /* table name */
        $YANA->setVar('PRIMARY_KEY', $primary_key); /* name of primary key column */
        $last_page = $this->database->length($table, $where); /* total number of rows */
        $YANA->setVar('LAST_PAGE', $last_page);
        $YANA->setVar('SORT', $sortBy); /* column name */

        /* deprecated */
        $YANA->setVar('WHERE', ''); /* the content of the where clause is not provided anymore */

        /* create a footer with links to other pages of the resultset */
        if ($last_page > ($entPerPage * 2)) {
            $liste = array();
            $dots = false;
            for($k = 0; $k < ceil($last_page / $entPerPage); $k++)
            {
                $it   = count($liste);
                $first = ($k * $entPerPage);
                if (($k + 1) * $entPerPage < $last_page) {
                    $last  = ($k + 1) * $entPerPage;
                } else {
                    $last  = $last_page;
                }
                if ($last_page > 10 * $entPerPage && $k > 1 && $k < ceil($last_page/$entPerPage) - 2 && !($first > $page - $entPerPage - 1 && $last < $page + 1 + (2 * $entPerPage))) {
                    if (!$dots) {
                        $dots = true;
                        $liste[$it]['TOO_MANY'] = true;
                    }
                } else {
                    $liste[$it]['FIRST'] = ($first + 1);
                    $liste[$it]['LAST']  = $last;
                    if ($first > $page-1 && $last < $page + $entPerPage + 1) {
                        $dots = false;
                    }    
                }
            } /* end foreach */

            /* provide result to template */
            $YANA->setVar('LIST_OF_ENTRIES', $liste);
        } else {
            /* only one page - footer is ignored */
            $YANA->setVar('LIST_OF_ENTRIES', array());
        }/* end if */

        $YANA->setVar('ENTRIES_PER_PAGE', $entPerPage); /* chunk size - numer of entries per page */
        $YANA->setVar('DESC', $desc); /* boolean value indicating sorting direction */
        /* number of the first result on the current page */
        if ($last_page > 0) {
            $YANA->setVar('FIRST_PAGE', $page + 1);
        } else {
            $YANA->setVar('FIRST_PAGE', 0); /* no entries on current page */
        }
        /* the numer of the last entry on the currently viewed page */
        if ($page + $entPerPage > $last_page) {
            $YANA->setVar('OFFSET_PAGE', $last_page); /* currenly viewed page is the last of the resultset */
        } else {
            $YANA->setVar('OFFSET_PAGE', $page + $entPerPage); /* there are more pages ... */
        }
        
        /* return an array containing the requested rows */
        return $result;
    }

    /**
     * _createArguments
     *
     * Call this to clean up user inputs before you call _getTable.
     *
     * @access  private
     * @param   array   $ARGS
     * @return  array
     * @ignore
     */
    function _createArguments($ARGS)
    {
        global $YANA;
        $profile_id = $YANA->session->getId();

        $args = array();
        $args['sort']  = "guestbook_date";
        $args['desc']  = true;
        $args['where'] = array(array("profile_id", $profile_id, "="));
        if (isset($ARGS['page']) && is_numeric($ARGS['page']) && $ARGS['page'] > 0) {
            $args['page']  = (int) $ARGS['page'];
        } else {
            $args['page']  = 0;
        }
        if (isset($ARGS['entries']) && is_numeric($ARGS['entries']) && $ARGS['entries'] > 0) {
            $args['entries'] = (int) $ARGS['entries'];
        } else {
            $args['entries'] = 10;
        }
        
        return $args;
    }

    /**
     * _securityCheck
     *
     * @access private
     * @return boolean
     * @ignore
     */
     function _securityCheck()
     {
        global $YANA;
        $id = strtolower($YANA->getVar('ID'));
        /* do not show new guestbooks, if Auto-Option is deactivated */
        $dir  = $YANA->registry->get('PROFILEDIR');
        $file = new inputStream($dir->getPath().$id.'.cfg');
        if ($id !== 'default' && !$file->exists() && !$YANA->getVar("PROFILE.AUTO")) {
            $YANA->report(new Log('Access restriction in effect. Access to undefined profile '.$id.' denied.'));
            $YANA->report(new Error('4041'));
            return false;
        } else {
            return true;
        }
    }

    /**
     * send mail
     *
     * @access  private
     * @param   string  $empfaenger
     * @param   string  $template
     * @param   array   $INPUT
     * @ignore
     */
    function _sendMail($empfaenger, $template, $INPUT)
    {
        assert('is_array($INPUT);');
        global $YANA;
        $INPUT = Hashtable::changeCase($INPUT,CASE_UPPER);
        $heute = getdate();
        $mail  = new mailer($template);
        $mail->subject = $YANA->getVar("LANGUAGE.MAIL_SUBJECT");
        $mail->insert("*", $INPUT);
        $mail->insert("*", $YANA->getVar("*"));
        $mail->insert('DATUM', $heute['mday'] . '.' . $heute['mon'] . '.' . $heute['year']);
        $mail->send($empfaenger);
    }

    /**
     * untaint input
     *
     * @access  private
     * @param   mixed  $empfaenger
     * @ignore
     */
    function _untaintInput(&$input)
    {
        if (is_array($input)) {
            foreach ($input as $a => $b)
            {
                $input[$a] = untaintInput($b, "string", 0, YANA_ESCAPE_USERTEXT);
            }
        }
    }

}

//  2007, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>