<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * IP-Blocker utility
 *
 * Security plugin that automatically blocks
 * banned IPs.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_ipblock extends Plugin
{

    /**#@+
     * @access  private
     * @var     blockfile
     * @ignore
     */

    var $whitelist = false;
    var $blacklist = false;

    /**#@-*/

    /**
     * Constructor
     *
     * @access  public
     * @param   string  $name a name for this plugin
     * @ignore
     */
    function plugin_ipblock($name)
    {
        settype($name,"string");
        $this->name = $name;
        global $YANA;
        $this->whitelist = $YANA->plugins->get($this->name, "IPWHITELIST");
        $this->blacklist = $YANA->plugins->get($this->name, "IPBLACKLIST");
        $dir             = $YANA->plugins->get($this->name, "BLOCKDIR");

        if (!$dir->exists() && !$dir->create(777)) {
            $YANA->report( new Log("Unable to create directory", $dir->getPath()) );
            $YANA->report( new Error("500"));
        }
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;

        if ($this->blacklist && $this->whitelist && $this->blacklist->exists()) {

            if ($this->whitelist->exists()) {
                $this->whitelist->read();
            }
            if ($this->blacklist->exists()) {
                $this->blacklist->read();
            }

            if ($this->blacklist->isBlocked() && ! ($this->whitelist->exists() && $this->whitelist->isBlocked())) {
                /* access denied */
                $YANA->report( new Error('block'));
                $YANA->exitTo(null); /* die */
                return false;
            } else {
                /* access granted */
                return true;
            }

        } else {
            return true;
        }
    }

}

//  2007, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>