<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * Singleton user administration implementation
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 * @ignore
 */
class user_admin extends Singleton
{

    /**
     * @access  private
     * @var     string
     */
    var $user_name = "";

    /**
     * Constructor
     *
     * @access public
     */
    function user_admin()
    {
        $this->checkSingletonConstraint();
        @session_start();
        global $YANA;
        if (isset($_SESSION['user_name'])) {
            $this->user_name = $_SESSION['user_name'];
        }    
        $YANA->setVar("SESSION_ID", session_id());
        $YANA->setVar("SESSION_NAME", session_name());
    }

    /**
     * @access public
     * @return boolean
     * @ignore
     */ 
    function isEmpty()
    {
        global $YANA;
        return $YANA->session->isEmpty();
    }

    /**
     * @access public
     * @return boolean 
     * @ignore
     */
     function get_user_configuration()
    {
       global $YANA;
       $YANA->setVar("BESCHREIBUNG", $YANA->getVar("LANGUAGE.DESCR_USER_CONFIGURATION"));

       $USERLIST   = $YANA->session->getUserList();
       $USERGROUPS = $YANA->session->getUserGroups();
       $YANA->setVar("USERLIST", $USERLIST);
       $YANA->setVar("USERGROUPS", $USERGROUPS);
       return true;
    }

    /**
     * @access  public
     * @param   string  $user_id
     * @param   bool    $original
     * @return  bool
     * @ignore
     */
    function get_user($user_id, $original = false)
    {
       settype($user_id, "string");
       settype($original, "boolean");
       global $YANA;

       $USER = $YANA->session->getUser($user_id);

       if (!$USER['ID'] || !$USER['ACTIVE']) {
           $YANA->report( new Error('user_unknown') );
           $YANA->exitTo('index');
           return false;
       } else {
           $YANA->setVar("USER", $USER);
           return true;
       }
    }

    /**
     * @access  public
     * @param   string  $user_id
     * @param   bool    $use_thumb
     * @return  bool
     * @ignore
     */
    function get_profile_image($user_id, $use_thumb = false)
    {
       settype($user_id, "string");
       global $YANA;

       $USER = $YANA->session->getUser($user_id);

       if (!$USER['ID'] || !$USER['ACTIVE']) {
           $YANA->report( new Error('user_unknown') );
           $YANA->exitTo('index');
           return false;
       } else {
           if (!empty($USER['USER_IMAGE']) && (strcasecmp($user_id, $this->user_name) === 0 || $USER['USER_IMAGE_ACTIVE'])) {
               if (!$use_thumb) {
                   $image = new Image($USER['USER_IMAGE']);
               } else {
                   $image = new Image(str_replace('/image.', '/thumb.', $USER['USER_IMAGE']));
               }
           } else {
               $image = new Image($YANA->getVar('DATADIR').'userpic.gif');
           }
           $image->outputToScreen();
           exit;
       }
    }

    /**
     * @access public
     * @param  string  $user
     * @return boolean
     * @ignore
     */
    function get_user_edit($user)
    {
       settype($user, "string");
       global $YANA;

       $YANA->setVar("BESCHREIBUNG", $YANA->getVar("LANGUAGE.DESCR_USER_EDIT"));

       $USER = $YANA->session->getUser($user);

       if (!$USER['ID']) {
           $YANA->report( new Error('user_unknown') );
           $YANA->exitTo('index');
           return false;
       } else {
           if (!empty($USER['MAIL'])) {
               $USER['MAIL'] = DisplayUtility::scanForAt($USER['MAIL']);
           }
           $USERGROUPS = $YANA->session->getUserGroups();
           $YANA->setVar("USER", $USER);
           $YANA->setVar("USERGROUPS",$USERGROUPS);
           return true;
       }
    }

    /**
     * @access public
     * @param  string  $user_name
     * @param  array   $INPUT
     * @return boolean
     * @ignore
     */
    function set_user_edit($user_name, $INPUT)
    {
        settype($user_name, "string");
        settype($INPUT, "array");
        global $YANA;

        $INPUT = Hashtable::changeCase($INPUT, CASE_UPPER);
        $REF = $this->_makeUser($user_name, $INPUT);

        if (!empty($INPUT['USER_IMAGE_ACTIVE'])) {
            $REF['USER_IMAGE_ACTIVE'] = true;
        } else {
            $REF['USER_IMAGE_ACTIVE'] = false;
        }

        if (isset($INPUT['PERMISSION'])) {
            $REF['PERMISSION'] = $INPUT['PERMISSION'];
        }
        settype($INPUT['ACTIVE'], "boolean");
        if (strtoupper($user_name) != "ADMINISTRATOR") {
            if (isset($INPUT['ACTIVE'])) {
                $REF['ACTIVE'] = $INPUT['ACTIVE'];
            }
            if (isset($INPUT['TYPE'])) {
                $REF['TYPE'] = $INPUT['TYPE'];
            }
        }

        if (!$YANA->session->insertUser($user_name, $REF)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * @access public
     * @param  array   $INPUT
     * @return boolean
     * @ignore
     */
    function set_profile_edit($INPUT)
    {
        settype($INPUT,"array");
        global $YANA;

        $INPUT = Hashtable::changeCase($INPUT,  CASE_UPPER);
        $FILES = array_change_key_case($_FILES, CASE_UPPER);

        /* untaint form input */
        foreach ($INPUT as $key => $element)
        {
            settype($element, "string");
            $element = strip_tags($element);
            $element = preg_replace("/\n/","[br]",$element);
            $element = preg_replace("/\s/"," ",$element);
            $INPUT[$key] = $element;
        } /* end foreach */
        unset($key);

        unset($INPUT["ACTIVE"]);
        unset($INPUT["USER_IMAGE_ACTIVE"]);
        unset($INPUT["PERM_GROUP"]);
        unset($INPUT["PERM_ID"]);
        unset($INPUT["PASSWORT"]);
        unset($INPUT["PASSWORD"]);
        unset($INPUT["TYPE"]);
        unset($INPUT["PERMISSION"]);
        unset($INPUT["ACTION"]);
        unset($INPUT[strtoupper(session_name())]);
        unset($INPUT["ID"]);
        /* end untaint input */

        $INPUT["MODIFIED"] = time();
        
        /* handle images */
        $id = 'USER_IMAGE';

        unset($INPUT[$id]);
        if (!empty($FILES[$id]['name'])) {
            
            $INPUT[$id]                 = $id;
            $INPUT['USER_IMAGE_ACTIVE'] = false;

        } /* end image handling */

        /* Begin: create user data */

        $USER = $YANA->session->getUser($this->user_name);

        /**
         * add code to get rid of old images
         *
         * Note: this is currently not necessary as every
         * user has a unique image resource associated with him.
         */

        return $YANA->session->insertUser($this->user_name, $INPUT);
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function set_user_delete($user_name)
    {
       settype($user_name,"string");
       global $YANA;

       if (strtoupper($this->user_name) == strtoupper($user_name)) {
           $YANA->report( new Alert('user_delete_self') );
           return false;
       } else if (strtoupper($user_name) == "ADMINISTRATOR") {
           $YANA->report( new Alert('ungueltige_eingabe') );
           return false;
       } else if (!$YANA->session->removeUser($user_name)) {
           $YANA->report( new Error('500') );
           return false;
       } else {
           $YANA->report( new Message('200') );
           return true;
       }
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function get_user_delete($user_name)
    {
       settype($user_name,"string");
       global $YANA;

       if (strtoupper($this->user_name) == strtoupper($user_name)) {
           $YANA->report( new Alert('user_delete_self') );
           $YANA->exitTo('index');
           return false;
       } else if (strtoupper($user_name) == "ADMINISTRATOR") {
           $YANA->report( new Alert('ungueltige_eingabe') );
           $YANA->exitTo('index');
           return false;
       } else {
           return true;
       }
    }
    
    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function set_user_pwd($user_name)
    {
       settype($user_name,"string");
       global $YANA;
       $YANA->language->read("user");

       $pwt = $YANA->session->setPwd($user_name);

       if (!$pwt) {
           return false;
       } else {
           $user = $YANA->session->getUser($user_name);
           $YANA->setVar('PASSWORT', $pwt);
           $YANA->setVar('NAME', $user['ID']);
           if (!empty($user['MAIL'])) {
               user_admin::sendMail($user['MAIL'], $YANA->skin->getFile("USER_PASSWORD_MAIL"));
           }
           return true;
       }
    }

    /**
     * @access public
     * @param  string   $user_name
     * @param  array    $INPUT
     * @return boolean
     * @ignore
     */
    function set_user_new()
    {
        settype($user_name,"string");
        settype($INPUT,"array");
        global $YANA;

        $NEW_USER = GuiCreator::getFormdata(GUI_NEW, new StructureFile('user'), "user");
        $user_name = $NEW_USER['id'];

        if (empty($user_name)) {
            $YANA->report( new Alert('ungueltige_eingabe') );
            return false;
        } else if ($this->userExists($user_name)) {
            $YANA->report( new Alert('user_already_exists') );
            return false;
        } else if (!$YANA->session->insertUser($user_name, $NEW_USER)) {
            $YANA->report( new Error('500') );
            return false;
        } else {
            return $this->set_user_pwd($user_name);
        }
    }

    /**
     * @access public
     * @param  string   $user_name
     * @param  array    $INPUT
     * @return boolean
     * @ignore
     */
    function user_authentification($user_name, $INPUT)
    {
       settype($user_name,"string");
       settype($INPUT,"array");
       global $YANA;

       $INPUT['ACTIVE'] = true;

       if (!$user_name) {
           $YANA->report( new Alert('ungueltige_eingabe') );
           $YANA->exitTo('');
           return false;
       } else if ($this->userExists($user_name)) {
           $YANA->report( new Alert('user_already_exists') );
           $YANA->exitTo('index');
           return false;
       } else {
           $INPUT = $this->_makeUser($user_name,$INPUT);
           $INPUT['PASSWORT'] = 'UNINITIALIZED';

           if (!$YANA->session->insertUser($user_name, $INPUT)) {
               $YANA->report( new Error('500') );
               $YANA->exitTo('');
               return false;
           } else {
               return $this->set_user_pwd($user_name);
           }
       }
    }

    /**
     * @access  public
     * @param   string  $user_name
     * @return  bool
     * @ignore
     */
    function userExists($user_name)
    {
        settype($user_name, "string");
        global $YANA;
        return $YANA->session->userExists($user_name);
    }

    /**
     * @access  private
     * @param   string  $user_name
     * @param   array   $INPUT
     * @return  array
     * @ignore
     */
    function _makeUser($user_name, $INPUT)
    {
        settype($INPUT, "array");

        /* untaint form input */
        foreach ($INPUT as $key => $element)
        {
            if (is_scalar($element)) {
                settype($element, "string");
                $element = strip_tags($element);
                $element = preg_replace("/\n/","[br]",$element);
                $element = preg_replace("/\s/"," ",$element);
                $INPUT[$key] = $element;
            }
        } /* end foreach */
        unset($key);

        $REF = array();
        $INPUT = Hashtable::changeCase($INPUT, CASE_UPPER);

        if (isset($INPUT['MAIL'])) {
            $REF['MAIL'] = $INPUT['MAIL'];
        }

        return $REF;
    }

    /**
     * @access  public
     * @static
     * @param   string  $empfaenger
     * @param   string  $template
     * @ignore
     */
    function sendMail($empfaenger, $template)
    {
        settype($empfaenger, "string");
        settype($template,   "string");

        global $YANA;
        $heute                = getdate();
        $mail                 = new mailer($template);
        $mail->subject        = $YANA->getVar("LANGUAGE.USER.MAIL_SUBJECT")."\n";
        $mail->insert('DATUM', $heute['mday'] . '.' . $heute['mon'] . '.' . $heute['year']);
        $mail->send($empfaenger);
    }

}

//  2007, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>