<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * user management plugin
 *
 * This creates forms and implements functions to
 * manage user data.
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_user_admin extends Plugin
{

    /**
     * @access private
     * @var user_admin 
     */
    var $file;

    /**
     * Constructor
     *
     * @param string $name a name for this plugin
     * @access public
     * @ignore
     */
    function plugin_user_admin($name) 
    {
        settype($name,"string");
        $this->name = $name;
        $this->file = new user_admin();
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        settype($event, "string");
        settype($ARGS, "array");
        global $YANA;

        if (!$this->file->isEmpty()) {

            switch ($event) {

                case "get_user_list":
                    return $this->file->get_user_configuration();
                break;

                case "get_user_edit":
                    $YANA->setVar("TARGET",$ARGS['target']);
                    return $this->file->get_user_edit($ARGS['target']);
                break;

                case "set_user_edit":
                    $YANA->setVar("TARGET",$ARGS['target']);
                    $ARGS['active'] = !empty($ARGS['active']);
                    if (isset($ARGS['permission_id']) && is_array($ARGS['permission_id']) && isset($ARGS['permission_group']) && is_array($ARGS['permission_group']) && count($ARGS['permission_id']) == count($ARGS['permission_group'])) {
                        $ARGS['permission'] = array();
                        for ($i = 0; $i < count($ARGS['permission_id']); $i++)
                        {
                            $key   = array_pop($ARGS['permission_id']);
                            $value = array_pop($ARGS['permission_group']);
                            if (!empty($key) && is_scalar($value)) {
                                $ARGS['permission'][$key] = $value;
                            }
                        }
                        unset($ARGS['permission_id'], $ARGS['permission_group']);
                    }
                    return $this->file->set_user_edit($ARGS['target'],$ARGS);
                break;

                case "get_user_delete":
                    $YANA->setVar("TARGET", $ARGS['target']);
                    return $this->file->get_user_delete($ARGS['target']);
                break;

                case "set_user_delete":
                    return $this->file->set_user_delete($ARGS['target']);
                break;

                case "set_user_pwd":
                    $YANA->language->read("user");
                    if ($this->file->set_user_pwd($ARGS['target']))
                    {
                        $YANA->report( new Message('200'));
                    } else {
                        $YANA->report( new Error('500'));
                    }
                    $YANA->exitTo('index');
                    return false;                     
                break;

                /* <FORMULAR> Mail-Authentifizierung */
                case "get_user_mail":
                    $YANA->setVar("TARGET",(!empty($ARGS['target'])) ? $ARGS['target'] : '');
                    if (!$YANA->getVar("PROFILE.USER.ALLOW_CREATE")) {
                        $YANA->report( new Error('ungueltige_eingabe'));
                        $YANA->exitTo('');
                        return false;
                    }

                    $YANA->setVar("ACTION_PARAMS", '<input type="hidden" name="action" value="SET_USER_MAIL">');
                break;

                /* <SESSION-ERZEUGEN> Mail-Authentifizierung */
                case "set_user_mail":
                    if (!$YANA->getVar("PROFILE.USER.ALLOW_CREATE")) {
                       $YANA->report( new error("ungueltige_eingabe"));
                       return false;
                    }

                    $db = $YANA->connect("user");
                    $mail = untaintInput(strtolower($ARGS['mail']), 'mail', 255);
                    $name = strtolower($ARGS['username']);

                    if (empty($name) || empty($mail)) {
                        $YANA->report( new warning("ungueltige_eingabe"));
                        return false;
                    }

                    $key = md5($mail);

                    if ($db->exists("user.$name")) {
                        $YANA->report( new warning("user_already_exists"));
                        return false;
                    }

                    $duplicates = $db->get("newuser.*.newuser_id", array(array("newuser_key",$key,'=')));
                    if (is_array($duplicates)) {
                       foreach ($duplicates as $newuser_id)
                       {
                           $db->remove("newuser.$newuser_id");
                       }
                    }
                    unset($duplicates);
                    $db->write();

                    /* limit = 24h = 86400sek */
                    $old_entries = $db->get("newuser");
                    $limit       = time() - 86400;
                    if (is_array($old_entries)) {
                        foreach ($old_entries as $row)
                        {
                            if ($row['NEWUSER_UTC'] < $limit) {
                                $db->remove("newuser.".$row['NEWUSER_ID']);
                            }
                        }
                    }
                    unset($old_entries);
                    $db->write();

                    if (!$db->insert("newuser.*", array("newuser_name" => $name, "newuser_key" => $key, "newuser_mail" => $mail))) {
                        $YANA->report( new warning("ungueltige_eingabe"));
                        return false;
                    }

                    if (!$db->write()) {
                        return false;
                    }

                    $YANA->setVar('WEBSITE_URL', $YANA->getVar("REFERER"));
                    $YANA->setVar('KEY', $key);
                    $YANA->setVar('MAIL', $mail);
                    user_admin::sendMail($mail, $YANA->skin->getFile("USER_CONFIRM_MAIL"));

                    return true;
                break;

                case "user_authentification":
                    if (!$YANA->getVar("PROFILE.USER.ALLOW_CREATE")) {
                        $YANA->report( new error("ungueltige_eingabe"));
                        return false;
                    }

                    $db = $YANA->connect("user");

                    if (!$db->exists('newuser',array(array('newuser_key', $ARGS['target'],'=')))) {
                        $YANA->report( new error("ungueltige_eingabe"));
                        return false;
                    } else {
                        $id = $db->get('newuser.*.newuser_id',array(array('newuser_key', $ARGS['target'],'=')));
                        $entry = $db->get("newuser.${id}");
                        $REF = $ARGS;
                        $REF['MAIL'] = $entry['NEWUSER_MAIL'];
                        if ($this->file->user_authentification($entry['NEWUSER_NAME'], $REF)) {
                            $YANA->report( new Alert('first_login'));
                            $YANA->exitTo('');
                            return true;
                        } else {
                            return false;
                        }
                    }
                break;

                case "view_profile":
                    return $this->file->get_user($ARGS['target']);
                break;

                case "get_profile_edit":
                    return $this->file->get_user($_SESSION['user_name'], true);
                break;

                case "set_profile_edit":
                    return $this->file->set_profile_edit($ARGS);
                break;

                case "get_profile_image":
                    if (!empty($ARGS['thumb'])) {
                        return $this->file->get_profile_image($ARGS['target'], true);
                    } else {
                        return $this->file->get_profile_image($ARGS['target'], false);
                    }
                break;

                default:
                    $YANA->language->read("user");
                break;

            } /* end switch */
        }
        return true;
    }

    /**
     * Create "new user" form
     *
     * this action expects no arguments
     *
     * on success:
     * ->  user::set_user_new
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function get_user_new($ARGS)
    {
        return true;
    }

    /**
     * Insert a new user
     *
     * handles input from action "get_user_new"
     * this action expects no arguments
     *
     * @access  public
     * @param   array    $ARGS
     * @return  bool
     */
    function set_user_new($ARGS)
    {
        global $YANA;
        $YANA->setVar('ID', $_POST['id']);
        return $this->file->set_user_new();
    }

}

//  2007, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>