/**
 * this file contains a simple AJAX-class
 *
 * @author      Thomas Meyer
 * @package     yana
 * @subpackage  utilities
 * @license     http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * constructor
 *
 * Purpose of this class is to send http-requests
 * to a server-side script and output the result
 * to an element in the current document,
 * identified by it's id.
 *
 * @param  string  url
 */
function AjaxRequest(url)
{
    if (typeof url == 'string') {
        this.url = url;
    } else {
        this.url = false;
        alert('Wrong argument type for argument 1 in constructor AjaxRequest. String expected, found "'+(typeof url)+'" instead.');
    }
    if (document.all){
        this.http = new ActiveXObject('Microsoft.XMLHTTP');
    } else {
        this.http = new XMLHttpRequest();
    }
    AjaxRequest.prototype.instance = this;
}

/**
 * send a request
 *
 * @access  public
 * @name    AjaxRequest::send()
 * @param   string  args  will be appended to url
 */
function AjaxRequest_send(args)
{
    if (!this.url) {
        alert('Cannot send request in AjaxRequest.send(). No target url specified.');
    } else {
        if (typeof args != 'string') {
            alert('Wrong argument type for argument 1 in method AjaxRequest.send(). String expected, found "'+(typeof args)+'" instead.');
            args = '';
        }
        var o = document.getElementById(this.id);
        if (o) {
            o.innerHTML = '<div class="ajax_loading">... Loading.</div>';
        }
        this.http.open('get', this.url + args, true);
        this.http.onreadystatechange = this.handle;
        this.http.send(null);
    }
}

/**
 * set the function to handle the response
 *
 * @access  public
 * @name    AjaxRequest::setHandler()
 * @param   function  newHandler
 */
function AjaxRequest_setHandler(newHandler)
{
    if (typeof newHandler != 'Function') {
        alert('Wrong argument type for argument 1 in method AjaxRequest.setHandler(). Function expected, found "'+(typeof newHandler)+'" instead.');
    } else {
        this.handle = newHandler;
    }
}

/**
 * set the id of the document node to ouput the response text to
 *
 * @access  public
 * @name    AjaxRequest::setTarget()
 * @param   string  id
 */
function AjaxRequest_setTarget(id)
{
    if ((typeof id) != 'string') {
        alert('Wrong argument type for argument 1 in method AjaxRequest.setTarget(). String expected, found "'+(typeof id)+'" instead.');
    } else if (!document.getElementById(id)) {
        alert('Invalid argument 1 in method AjaxRequest.setTarget(). There is no element with the id "'+id+'" within the current document.');
    } else {
        this.id = id;
    }
}

/**
 * handle the response
 *
 * @access  private
 * @name    AjaxRequest::handle()
 */
function AjaxRequest_handle()
{
    var o = AjaxRequest.prototype.instance;
    if (o.http.readyState == 4) {
        var o1 = document.getElementById(o.id);
        if (o1) {
            o1.innerHTML = o.http.responseText;
        }
    }
}

AjaxRequest.prototype.send = AjaxRequest_send;
AjaxRequest.prototype.handle = AjaxRequest_handle;
AjaxRequest.prototype.setHandler = AjaxRequest_setHandler;
AjaxRequest.prototype.setTarget = AjaxRequest_setTarget;

/******************************************
  Thomas Meyer www.All-Community.de
 Copyright Hinweis nicht entfernen!
******************************************/